import subprocess
import unittest

import mock
import netifaces

import charmhelpers.contrib.network.ip as net_ip
from mock import patch, MagicMock

import nose.tools
import six

if not six.PY3:
    builtin_open = '__builtin__.open'
    builtin_import = '__builtin__.__import__'
else:
    builtin_open = 'builtins.open'
    builtin_import = 'builtins.__import__'

DUMMY_ADDRESSES = {
    'lo': {
        17: [{'peer': '00:00:00:00:00:00',
              'addr': '00:00:00:00:00:00'}],
        2: [{'peer': '127.0.0.1', 'netmask':
             '255.0.0.0', 'addr': '127.0.0.1'}],
        10: [{'netmask': 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff',
              'addr': '::1'}]
    },
    'eth0': {
        2: [{'addr': '192.168.1.55',
             'broadcast': '192.168.1.255',
             'netmask': '255.255.255.0'}],
        10: [{'addr': '2a01:348:2f4:0:685e:5748:ae62:209f',
              'netmask': 'ffff:ffff:ffff:ffff::'},
             {'addr': 'fe80::3e97:eff:fe8b:1cf7%eth0',
              'netmask': 'ffff:ffff:ffff:ffff::'},
             {'netmask': 'ffff:ffff:ffff:ffff::/64',
              'addr': 'fd2d:dec4:cf59:3c16::1'},
             {'addr': '2001:db8:1::',
              'netmask': 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff'}],
        17: [{'addr': '3c:97:0e:8b:1c:f7',
              'broadcast': 'ff:ff:ff:ff:ff:ff'}]
    },
    'eth0:1': {
        2: [{'addr': '192.168.1.56',
             'broadcast': '192.168.1.255',
             'netmask': '255.255.255.0'}],
    },
    'eth1': {
        2: [{'addr': '10.5.0.1',
             'broadcast': '10.5.255.255',
             'netmask': '255.255.0.0'},
            {'addr': '10.6.0.2',
             'broadcast': '10.6.0.255',
             'netmask': '255.255.255.0'}],
        3: [{'addr': 'fe80::3e97:eff:fe8b:1cf7%eth1',
             'netmask': 'ffff:ffff:ffff:ffff::'}],
        17: [{'addr': '3c:97:0e:8b:1c:f7',
              'broadcast': 'ff:ff:ff:ff:ff:ff'}]
    },
    'eth2': {
        10: [{'addr': '3a01:348:2f4:0:685e:5748:ae62:209f',
              'netmask': 'ffff:ffff:ffff:ffff::'},
             {'addr': 'fe80::3e97:edd:fe8b:1cf7%eth0',
              'netmask': 'ffff:ffff:ffff:ffff::'}],
        17: [{'addr': '3c:97:0e:8b:1c:f7',
              'broadcast': 'ff:ff:ff:ff:ff:ff'}]
    },
    'eth2:1': {
        2: [{'addr': '192.168.10.58',
             'broadcast': '192.168.1.255',
             'netmask': '255.255.255.0'}],
    },
}

IP_OUTPUT = b"""link/ether fa:16:3e:2a:cc:ce brd ff:ff:ff:ff:ff:ff
    inet 10.5.16.93/16 brd 10.5.255.255 scope global eth0
       valid_lft forever preferred_lft forever
    inet6 2001:db8:1:0:d0cf:528c:23eb:6000/64 scope global
       valid_lft forever preferred_lft forever
    inet6 2001:db8:1:0:2918:3444:852:5b8a/64 scope global temporary dynamic
       valid_lft 86400sec preferred_lft 14400sec
    inet6 2001:db8:1:0:f816:3eff:fe2a:ccce/64 scope global dynamic
       valid_lft 86400sec preferred_lft 14400sec
    inet6 fe80::f816:3eff:fe2a:ccce/64 scope link
       valid_lft forever preferred_lft forever
"""

IP2_OUTPUT = b"""link/ether fa:16:3e:2a:cc:ce brd ff:ff:ff:ff:ff:ff
    inet 10.5.16.93/16 brd 10.5.255.255 scope global eth0
       valid_lft forever preferred_lft forever
    inet6 2001:db8:1:0:d0cf:528c:23eb:6000/64 scope global
       valid_lft forever preferred_lft forever
    inet6 2001:db8:1:0:2918:3444:852:5b8a/64 scope global temporary dynamic
       valid_lft 86400sec preferred_lft 14400sec
    inet6 2001:db8:1:0:f816:3eff:fe2a:ccce/64 scope global mngtmpaddr dynamic
       valid_lft 86400sec preferred_lft 14400sec
    inet6 fe80::f816:3eff:fe2a:ccce/64 scope link
       valid_lft forever preferred_lft forever
"""

IP_OUTPUT_NO_VALID = b"""link/ether fa:16:3e:2a:cc:ce brd ff:ff:ff:ff:ff:ff
    inet 10.5.16.93/16 brd 10.5.255.255 scope global eth0
       valid_lft forever preferred_lft forever
    inet6 2001:db8:1:0:2918:3444:852:5b8a/64 scope global temporary dynamic
       valid_lft 86400sec preferred_lft 14400sec
    inet6 fe80::f816:3eff:fe2a:ccce/64 scope link
       valid_lft forever preferred_lft forever
"""


class FakeAnswer(object):
    def __init__(self, ip):
        self.ip = ip

    def __str__(self):
        return self.ip


class FakeResolver(object):
    def __init__(self, ip):
        self.ip = ip

    def query(self, hostname, query_type):
        if self.ip == '':
            return []
        else:
            return [FakeAnswer(self.ip)]


class FakeReverse(object):
    def from_address(self, address):
        return '156.94.189.91.in-addr.arpa'


class FakeDNSName(object):
    def __init__(self, dnsname):
        pass


class FakeDNS(object):
    def __init__(self, ip):
        self.resolver = FakeResolver(ip)
        self.reversename = FakeReverse()
        self.name = MagicMock()
        self.name.Name = FakeDNSName


class IPTest(unittest.TestCase):

    def mock_ifaddresses(self, iface):
        return DUMMY_ADDRESSES[iface]

    def test_get_address_in_network_with_invalid_net(self):
        for net in ['192.168.300/22', '192.168.1.0/2a', '2.a']:
            self.assertRaises(ValueError,
                              net_ip.get_address_in_network,
                              net)

    def _test_get_address_in_network(self, expect_ip_addr,
                                     network, fallback=None, fatal=False):

        def side_effect(iface):
            return DUMMY_ADDRESSES[iface]

        with mock.patch.object(netifaces, 'interfaces') as interfaces:
            interfaces.return_value = sorted(DUMMY_ADDRESSES.keys())
            with mock.patch.object(netifaces, 'ifaddresses') as ifaddresses:
                ifaddresses.side_effect = side_effect
                if not fatal:
                    self.assertEqual(expect_ip_addr,
                                     net_ip.get_address_in_network(network,
                                                                   fallback,
                                                                   fatal))
                else:
                    net_ip.get_address_in_network(network, fallback, fatal)

    @mock.patch.object(subprocess, 'call')
    def test_get_address_in_network_with_none(self, popen):
        fallback = '10.10.10.10'
        self.assertEqual(fallback,
                         net_ip.get_address_in_network(None, fallback))
        self.assertEqual(None,
                         net_ip.get_address_in_network(None))

        self.assertRaises(ValueError, self._test_get_address_in_network,
                          None, None, fatal=True)

    def test_get_address_in_network_ipv4(self):
        self._test_get_address_in_network('192.168.1.55', '192.168.1.0/24')

    def test_get_address_in_network_ipv4_multi(self):
        # Assumes that there is an address configured on both but the first
        # one is picked#
        self._test_get_address_in_network('192.168.1.55',
                                          '192.168.1.0/24 192.168.10.0/24')

    def test_get_address_in_network_ipv4_multi2(self):
        # Assumes that there is nothing configured on 192.168.11.0/24
        self._test_get_address_in_network('192.168.10.58',
                                          '192.168.11.0/24 192.168.10.0/24')

    def test_get_address_in_network_ipv4_secondary(self):
        self._test_get_address_in_network('10.6.0.2',
                                          '10.6.0.0/24')

    def test_get_address_in_network_ipv6(self):
        self._test_get_address_in_network('2a01:348:2f4:0:685e:5748:ae62:209f',
                                          '2a01:348:2f4::/64')

    def test_get_address_in_network_with_non_existent_net(self):
        self._test_get_address_in_network(None, '172.16.0.0/16')

    def test_get_address_in_network_fallback_works(self):
        fallback = '10.10.0.0'
        self._test_get_address_in_network(fallback, '172.16.0.0/16', fallback)

    @mock.patch.object(subprocess, 'call')
    def test_get_address_in_network_not_found_fatal(self, popen):
        self.assertRaises(ValueError, self._test_get_address_in_network,
                          None, '172.16.0.0/16', fatal=True)

    def test_get_address_in_network_not_found_not_fatal(self):
        self._test_get_address_in_network(None, '172.16.0.0/16', fatal=False)

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_address_in_network_netmask(self, _interfaces, _ifaddresses):
        """
        Validates that get_address_in_network works with a netmask
        that uses the format 'ffff:ffff:ffff::/prefixlen'
        """
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        self._test_get_address_in_network('fd2d:dec4:cf59:3c16::1',
                                          'fd2d:dec4:cf59:3c16::/64',
                                          fatal=False)

    def test_is_address_in_network(self):
        self.assertTrue(
            net_ip.is_address_in_network(
                '192.168.1.0/24',
                '192.168.1.1'))
        self.assertFalse(
            net_ip.is_address_in_network(
                '192.168.1.0/24',
                '10.5.1.1'))
        self.assertRaises(ValueError, net_ip.is_address_in_network,
                          'broken', '192.168.1.1')
        self.assertRaises(ValueError, net_ip.is_address_in_network,
                          '192.168.1.0/24', 'hostname')
        self.assertTrue(
            net_ip.is_address_in_network(
                '2a01:348:2f4::/64',
                '2a01:348:2f4:0:685e:5748:ae62:209f')
        )
        self.assertFalse(
            net_ip.is_address_in_network(
                '2a01:348:2f4::/64',
                'fdfc:3bd5:210b:cc8d:8c80:9e10:3f07:371')
        )

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_iface_for_address(self, _interfaces, _ifaddresses):
        def mock_ifaddresses(iface):
            return DUMMY_ADDRESSES[iface]
        _interfaces.return_value = ['eth0', 'eth1']
        _ifaddresses.side_effect = mock_ifaddresses
        self.assertEquals(
            net_ip.get_iface_for_address('192.168.1.220'),
            'eth0')
        self.assertEquals(net_ip.get_iface_for_address('10.5.20.4'), 'eth1')
        self.assertEquals(
            net_ip.get_iface_for_address('2a01:348:2f4:0:685e:5748:ae62:210f'),
            'eth0'
        )
        self.assertEquals(net_ip.get_iface_for_address('172.4.5.5'), None)

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_netmask_for_address(self, _interfaces, _ifaddresses):
        def mock_ifaddresses(iface):
            return DUMMY_ADDRESSES[iface]
        _interfaces.return_value = ['eth0', 'eth1']
        _ifaddresses.side_effect = mock_ifaddresses
        self.assertEquals(
            net_ip.get_netmask_for_address('192.168.1.220'),
            '255.255.255.0')
        self.assertEquals(
            net_ip.get_netmask_for_address('10.5.20.4'),
            '255.255.0.0')
        self.assertEquals(net_ip.get_netmask_for_address('172.4.5.5'), None)
        self.assertEquals(
            net_ip.get_netmask_for_address(
                '2a01:348:2f4:0:685e:5748:ae62:210f'),
            '64'
        )
        self.assertEquals(
            net_ip.get_netmask_for_address('2001:db8:1::'),
            '128'
        )

    def test_is_ipv6(self):
        self.assertFalse(net_ip.is_ipv6('myhost'))
        self.assertFalse(net_ip.is_ipv6('172.4.5.5'))
        self.assertTrue(net_ip.is_ipv6('2a01:348:2f4:0:685e:5748:ae62:209f'))

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_ipv6_addr_no_ipv6(self, _interfaces, _ifaddresses):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        with nose.tools.assert_raises(Exception):
            net_ip.get_ipv6_addr('eth0:1')

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_ipv6_addr_no_global_ipv6(self, _interfaces,
                                          _ifaddresses):
        DUMMY_ADDRESSES = {
            'eth0': {
                10: [{'addr': 'fe80::3e97:eff:fe8b:1cf7%eth0',
                      'netmask': 'ffff:ffff:ffff:ffff::'}],
            }
        }
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        self.assertRaises(Exception, net_ip.get_ipv6_addr)

    @patch('charmhelpers.contrib.network.ip.get_iface_from_addr')
    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_ipv6_addr_exc_list(self, _interfaces, _ifaddresses,
                                    mock_get_iface_from_addr):
        def mock_ifaddresses(iface):
            return DUMMY_ADDRESSES[iface]

        _interfaces.return_value = ['eth0', 'eth1']
        _ifaddresses.side_effect = mock_ifaddresses

        result = net_ip.get_ipv6_addr(
            exc_list='2a01:348:2f4:0:685e:5748:ae62:209f',
            inc_aliases=True,
            fatal=False
        )
        self.assertEqual([], result)

    @patch('charmhelpers.contrib.network.ip.get_iface_from_addr')
    @patch('charmhelpers.contrib.network.ip.subprocess.check_output')
    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_ipv6_addr(self, _interfaces, _ifaddresses, mock_check_out,
                           mock_get_iface_from_addr):
        mock_get_iface_from_addr.return_value = 'eth0'
        mock_check_out.return_value = \
            b"inet6 2a01:348:2f4:0:685e:5748:ae62:209f/64 scope global dynamic"
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        result = net_ip.get_ipv6_addr(dynamic_only=False)
        self.assertEqual(['2a01:348:2f4:0:685e:5748:ae62:209f'], result)

    @patch('charmhelpers.contrib.network.ip.get_iface_from_addr')
    @patch('charmhelpers.contrib.network.ip.subprocess.check_output')
    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_ipv6_addr_global_dynamic(self, _interfaces, _ifaddresses,
                                          mock_check_out,
                                          mock_get_iface_from_addr):
        mock_get_iface_from_addr.return_value = 'eth0'
        mock_check_out.return_value = \
            b"inet6 2a01:348:2f4:0:685e:5748:ae62:209f/64 scope global dynamic"
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        result = net_ip.get_ipv6_addr(dynamic_only=False)
        self.assertEqual(['2a01:348:2f4:0:685e:5748:ae62:209f'], result)

    @patch.object(netifaces, 'interfaces')
    def test_get_ipv6_addr_invalid_nic(self, _interfaces):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        self.assertRaises(Exception, net_ip.get_ipv6_addr, 'eth1')

    @patch('charmhelpers.contrib.network.ip.subprocess.check_output')
    def test_is_ipv6_disabled(self, mock_check_output):
        # verify that the function does look for the right thing
        mock_check_output.return_value = """
        Some lines before
        net.ipv6.conf.all.disable_ipv6 = 1
        Some lines afterward
        """
        self.assertTrue(net_ip.is_ipv6_disabled())
        mock_check_output.assert_called_once_with(
            ['sysctl', 'net.ipv6.conf.all.disable_ipv6'],
            stderr=subprocess.STDOUT, universal_newlines=True)
        # if it isn't there, it must return false
        mock_check_output.return_value = ""
        self.assertFalse(net_ip.is_ipv6_disabled())
        # If the syscall returns an error, then return True

        def fake_check_call(*args, **kwargs):
            raise subprocess.CalledProcessError(['called'], 1)
        mock_check_output.side_effect = fake_check_call
        self.assertTrue(net_ip.is_ipv6_disabled())

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_iface_addr(self, _interfaces, _ifaddresses):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        result = net_ip.get_iface_addr("eth0")
        self.assertEqual(["192.168.1.55"], result)

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_iface_addr_excaliases(self, _interfaces, _ifaddresses):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        result = net_ip.get_iface_addr("eth0")
        self.assertEqual(['192.168.1.55'], result)

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_iface_addr_incaliases(self, _interfaces, _ifaddresses):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        result = net_ip.get_iface_addr("eth0", inc_aliases=True)
        self.assertEqual(['192.168.1.55', '192.168.1.56'], result)

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_iface_addr_exclist(self, _interfaces, _ifaddresses):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        result = net_ip.get_iface_addr("eth0", inc_aliases=True,
                                       exc_list=['192.168.1.55'])
        self.assertEqual(['192.168.1.56'], result)

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_iface_addr_mixedaddr(self, _interfaces, _ifaddresses):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        result = net_ip.get_iface_addr("eth2", inc_aliases=True)
        self.assertEqual(["192.168.10.58"], result)

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_iface_addr_full_interface_path(self, _interfaces,
                                                _ifaddresses):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__
        result = net_ip.get_iface_addr("/dev/eth0")
        self.assertEqual(["192.168.1.55"], result)

    @patch.object(netifaces, 'interfaces')
    def test_get_iface_addr_invalid_type(self, _interfaces):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        with nose.tools.assert_raises(Exception):
            net_ip.get_iface_addr(iface='eth0', inet_type='AF_BOB')

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_iface_addr_invalid_interface(self, _interfaces, _ifaddresses):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        result = net_ip.get_ipv4_addr("eth3", fatal=False)
        self.assertEqual([], result)

    @patch.object(netifaces, 'interfaces')
    def test_get_iface_addr_invalid_interface_fatal(self, _interfaces):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        with nose.tools.assert_raises(Exception):
            net_ip.get_ipv4_addr("eth3", fatal=True)

    @patch.object(netifaces, 'interfaces')
    def test_get_iface_addr_invalid_interface_fatal_incaliases(self,
                                                               _interfaces):
        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        with nose.tools.assert_raises(Exception):
            net_ip.get_ipv4_addr("eth3", fatal=True, inc_aliases=True)

    @patch.object(netifaces, 'ifaddresses')
    @patch.object(netifaces, 'interfaces')
    def test_get_get_iface_addr_interface_has_no_ipv4(self, _interfaces,
                                                      _ifaddresses):

        # This will raise a KeyError since we are looking for "2"
        # (actally, netiface.AF_INET).
        DUMMY_ADDRESSES = {
            'eth0': {
                10: [{'addr': 'fe80::3e97:eff:fe8b:1cf7%eth0',
                      'netmask': 'ffff:ffff:ffff:ffff::'}],
            }
        }

        _interfaces.return_value = DUMMY_ADDRESSES.keys()
        _ifaddresses.side_effect = DUMMY_ADDRESSES.__getitem__

        result = net_ip.get_ipv4_addr("eth0", fatal=False)
        self.assertEqual([], result)

    @patch('glob.glob')
    def test_get_bridges(self, _glob):
        _glob.return_value = ['/sys/devices/virtual/net/br0/bridge']
        self.assertEqual(['br0'], net_ip.get_bridges())

    @patch.object(net_ip, 'get_bridges')
    @patch('glob.glob')
    def test_get_bridge_nics(self, _glob, _get_bridges):
        _glob.return_value = ['/sys/devices/virtual/net/br0/brif/eth4',
                              '/sys/devices/virtual/net/br0/brif/eth5']
        self.assertEqual(['eth4', 'eth5'], net_ip.get_bridge_nics('br0'))

    @patch.object(net_ip, 'get_bridges')
    @patch('glob.glob')
    def test_get_bridge_nics_invalid_br(self, _glob, _get_bridges):
        _glob.return_value = []
        self.assertEqual([], net_ip.get_bridge_nics('br1'))

    @patch.object(net_ip, 'get_bridges')
    @patch.object(net_ip, 'get_bridge_nics')
    def test_is_bridge_member(self, _get_bridge_nics, _get_bridges):
        _get_bridges.return_value = ['br0']
        _get_bridge_nics.return_value = ['eth4', 'eth5']
        self.assertTrue(net_ip.is_bridge_member('eth4'))
        self.assertFalse(net_ip.is_bridge_member('eth6'))

    def test_format_ipv6_addr(self):
        DUMMY_ADDRESS = '2001:db8:1:0:f131:fc84:ea37:7d4'
        self.assertEquals(net_ip.format_ipv6_addr(DUMMY_ADDRESS),
                          '[2001:db8:1:0:f131:fc84:ea37:7d4]')

    def test_format_invalid_ipv6_addr(self):
        INVALID_IPV6_ADDR = 'myhost'
        self.assertEquals(net_ip.format_ipv6_addr(INVALID_IPV6_ADDR),
                          None)

    @patch('charmhelpers.contrib.network.ip.get_iface_from_addr')
    @patch('charmhelpers.contrib.network.ip.subprocess.check_output')
    @patch('charmhelpers.contrib.network.ip.get_iface_addr')
    def test_get_ipv6_global_address(self, mock_get_iface_addr, mock_check_out,
                                     mock_get_iface_from_addr):
        mock_get_iface_from_addr.return_value = 'eth0'
        mock_check_out.return_value = IP_OUTPUT
        scope_global_addr = '2001:db8:1:0:d0cf:528c:23eb:6000'
        scope_global_dyn_addr = '2001:db8:1:0:f816:3eff:fe2a:ccce'
        mock_get_iface_addr.return_value = [scope_global_addr,
                                            scope_global_dyn_addr,
                                            '2001:db8:1:0:2918:3444:852:5b8a',
                                            'fe80::f816:3eff:fe2a:ccce%eth0']
        self.assertEqual([scope_global_addr, scope_global_dyn_addr],
                         net_ip.get_ipv6_addr(dynamic_only=False))

    @patch('charmhelpers.contrib.network.ip.get_iface_from_addr')
    @patch('charmhelpers.contrib.network.ip.subprocess.check_output')
    @patch('charmhelpers.contrib.network.ip.get_iface_addr')
    def test_get_ipv6_global_dynamic_address(self, mock_get_iface_addr,
                                             mock_check_out,
                                             mock_get_iface_from_addr):
        mock_get_iface_from_addr.return_value = 'eth0'
        mock_check_out.return_value = IP_OUTPUT
        scope_global_addr = '2001:db8:1:0:d0cf:528c:23eb:6000'
        scope_global_dyn_addr = '2001:db8:1:0:f816:3eff:fe2a:ccce'
        mock_get_iface_addr.return_value = [scope_global_addr,
                                            scope_global_dyn_addr,
                                            '2001:db8:1:0:2918:3444:852:5b8a',
                                            'fe80::f816:3eff:fe2a:ccce%eth0']
        self.assertEqual([scope_global_dyn_addr], net_ip.get_ipv6_addr())

    @patch('charmhelpers.contrib.network.ip.get_iface_from_addr')
    @patch('charmhelpers.contrib.network.ip.subprocess.check_output')
    @patch('charmhelpers.contrib.network.ip.get_iface_addr')
    def test_get_ipv6_global_dynamic_address_ip2(self, mock_get_iface_addr,
                                                 mock_check_out,
                                                 mock_get_iface_from_addr):
        mock_get_iface_from_addr.return_value = 'eth0'
        mock_check_out.return_value = IP2_OUTPUT
        scope_global_addr = '2001:db8:1:0:d0cf:528c:23eb:6000'
        scope_global_dyn_addr = '2001:db8:1:0:f816:3eff:fe2a:ccce'
        mock_get_iface_addr.return_value = [scope_global_addr,
                                            scope_global_dyn_addr,
                                            '2001:db8:1:0:2918:3444:852:5b8a',
                                            'fe80::f816:3eff:fe2a:ccce%eth0']
        self.assertEqual([scope_global_dyn_addr], net_ip.get_ipv6_addr())

    @patch('charmhelpers.contrib.network.ip.subprocess.check_output')
    @patch('charmhelpers.contrib.network.ip.get_iface_addr')
    def test_get_ipv6_global_dynamic_address_invalid_address(
            self, mock_get_iface_addr, mock_check_out):
        mock_get_iface_addr.return_value = []
        with nose.tools.assert_raises(Exception):
            net_ip.get_ipv6_addr()

        mock_get_iface_addr.return_value = ['2001:db8:1:0:2918:3444:852:5b8a']
        mock_check_out.return_value = IP_OUTPUT_NO_VALID
        with nose.tools.assert_raises(Exception):
            net_ip.get_ipv6_addr()

    @patch('charmhelpers.contrib.network.ip.get_iface_addr')
    def test_get_ipv6_addr_w_iface(self, mock_get_iface_addr):
        mock_get_iface_addr.return_value = []
        net_ip.get_ipv6_addr(iface='testif', fatal=False)
        mock_get_iface_addr.assert_called_once_with(iface='testif',
                                                    inet_type='AF_INET6',
                                                    inc_aliases=False,
                                                    fatal=False, exc_list=None)

    @patch('charmhelpers.contrib.network.ip.unit_get')
    @patch('charmhelpers.contrib.network.ip.get_iface_from_addr')
    @patch('charmhelpers.contrib.network.ip.get_iface_addr')
    def test_get_ipv6_addr_no_iface(self, mock_get_iface_addr,
                                    mock_get_iface_from_addr, mock_unit_get):
        mock_unit_get.return_value = '1.2.3.4'
        mock_get_iface_addr.return_value = []
        mock_get_iface_from_addr.return_value = "testif"
        net_ip.get_ipv6_addr(fatal=False)
        mock_get_iface_from_addr.assert_called_once_with('1.2.3.4')
        mock_get_iface_addr.assert_called_once_with(iface='testif',
                                                    inet_type='AF_INET6',
                                                    inc_aliases=False,
                                                    fatal=False, exc_list=None)

    @patch('netifaces.interfaces')
    @patch('netifaces.ifaddresses')
    @patch('charmhelpers.contrib.network.ip.log')
    def test_get_iface_from_addr(self, mock_log, mock_ifaddresses,
                                 mock_interfaces):
        mock_ifaddresses.side_effect = lambda iface: DUMMY_ADDRESSES[iface]
        mock_interfaces.return_value = sorted(DUMMY_ADDRESSES.keys())
        addr = 'fe80::3e97:eff:fe8b:1cf7'
        self.assertEqual(net_ip.get_iface_from_addr(addr), 'eth0')

        with nose.tools.assert_raises(Exception):
            net_ip.get_iface_from_addr('1.2.3.4')

    def test_is_ip(self):
        self.assertTrue(net_ip.is_ip('10.0.0.1'))
        self.assertTrue(net_ip.is_ip('2001:db8:1:0:2918:3444:852:5b8a'))
        self.assertFalse(net_ip.is_ip('www.ubuntu.com'))

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_host_ip_with_hostname(self, apt_install):
        fake_dns = FakeDNS('10.0.0.1')
        with patch(builtin_import, side_effect=[fake_dns]):
            ip = net_ip.get_host_ip('www.ubuntu.com')
        self.assertEquals(ip, '10.0.0.1')

    @patch('charmhelpers.contrib.network.ip.ns_query')
    @patch('charmhelpers.contrib.network.ip.socket.gethostbyname')
    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_host_ip_with_hostname_no_dns(self, apt_install, socket,
                                              ns_query):
        ns_query.return_value = []
        fake_dns = FakeDNS(None)
        socket.return_value = '10.0.0.1'
        with patch(builtin_import, side_effect=[fake_dns]):
            ip = net_ip.get_host_ip('www.ubuntu.com')
        self.assertEquals(ip, '10.0.0.1')

    @patch('charmhelpers.contrib.network.ip.log')
    @patch('charmhelpers.contrib.network.ip.ns_query')
    @patch('charmhelpers.contrib.network.ip.socket.gethostbyname')
    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_host_ip_with_hostname_fallback(self, apt_install, socket,
                                                ns_query, *args):
        ns_query.return_value = []
        fake_dns = FakeDNS(None)

        def r():
            raise Exception()

        socket.side_effect = r
        with patch(builtin_import, side_effect=[fake_dns]):
            ip = net_ip.get_host_ip('www.ubuntu.com', fallback='127.0.0.1')
        self.assertEquals(ip, '127.0.0.1')

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_host_ip_with_ip(self, apt_install):
        fake_dns = FakeDNS('5.5.5.5')
        with patch(builtin_import, side_effect=[fake_dns]):
            ip = net_ip.get_host_ip('4.2.2.1')
        self.assertEquals(ip, '4.2.2.1')

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_ns_query_trigger_apt_install(self, apt_install):
        fake_dns = FakeDNS('5.5.5.5')
        with patch(builtin_import, side_effect=[ImportError, fake_dns]):
            nsq = net_ip.ns_query('5.5.5.5')
            if six.PY2:
                apt_install.assert_called_with('python-dnspython', fatal=True)
            else:
                apt_install.assert_called_with('python3-dnspython', fatal=True)
        self.assertEquals(nsq, '5.5.5.5')

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_ns_query_ptr_record(self, apt_install):
        fake_dns = FakeDNS('127.0.0.1')
        with patch(builtin_import, side_effect=[fake_dns]):
            nsq = net_ip.ns_query('127.0.0.1')
        self.assertEquals(nsq, '127.0.0.1')

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_ns_query_a_record(self, apt_install):
        fake_dns = FakeDNS('127.0.0.1')
        fake_dns_name = FakeDNSName('www.somedomain.tld')
        with patch(builtin_import, side_effect=[fake_dns]):
            nsq = net_ip.ns_query(fake_dns_name)
        self.assertEquals(nsq, '127.0.0.1')

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_ns_query_blank_record(self, apt_install):
        fake_dns = FakeDNS(None)
        with patch(builtin_import, side_effect=[fake_dns, fake_dns]):
            nsq = net_ip.ns_query(None)
        self.assertEquals(nsq, None)

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_ns_query_lookup_fail(self, apt_install):
        fake_dns = FakeDNS('')
        with patch(builtin_import, side_effect=[fake_dns, fake_dns]):
            nsq = net_ip.ns_query('nonexistant')
        self.assertEquals(nsq, None)

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_hostname_with_ip(self, apt_install):
        fake_dns = FakeDNS('www.ubuntu.com')
        with patch(builtin_import, side_effect=[fake_dns, fake_dns]):
            hn = net_ip.get_hostname('4.2.2.1')
        self.assertEquals(hn, 'www.ubuntu.com')

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_hostname_with_ip_not_fqdn(self, apt_install):
        fake_dns = FakeDNS('packages.ubuntu.com')
        with patch(builtin_import, side_effect=[fake_dns, fake_dns]):
            hn = net_ip.get_hostname('4.2.2.1', fqdn=False)
        self.assertEquals(hn, 'packages')

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_hostname_with_hostname(self, apt_install):
        hn = net_ip.get_hostname('www.ubuntu.com')
        self.assertEquals(hn, 'www.ubuntu.com')

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_hostname_with_hostname_trailingdot(self, apt_install):
        hn = net_ip.get_hostname('www.ubuntu.com.')
        self.assertEquals(hn, 'www.ubuntu.com')

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_hostname_with_hostname_not_fqdn(self, apt_install):
        hn = net_ip.get_hostname('packages.ubuntu.com', fqdn=False)
        self.assertEquals(hn, 'packages')

    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_hostname_trigger_apt_install(self, apt_install):
        fake_dns = FakeDNS('www.ubuntu.com')
        with patch(builtin_import, side_effect=[ImportError, fake_dns,
                                                fake_dns]):
            hn = net_ip.get_hostname('4.2.2.1')
            if six.PY2:
                apt_install.assert_called_with('python-dnspython', fatal=True)
            else:
                apt_install.assert_called_with('python3-dnspython', fatal=True)

        self.assertEquals(hn, 'www.ubuntu.com')

    @patch('charmhelpers.contrib.network.ip.socket.gethostbyaddr')
    @patch('charmhelpers.contrib.network.ip.ns_query')
    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_hostname_lookup_fail(self, apt_install, ns_query, socket):
        fake_dns = FakeDNS('www.ubuntu.com')
        ns_query.return_value = []
        socket.return_value = ()
        with patch(builtin_import, side_effect=[fake_dns, fake_dns]):
            hn = net_ip.get_hostname('4.2.2.1')
        self.assertEquals(hn, None)

    @patch('charmhelpers.contrib.network.ip.socket.gethostbyaddr')
    @patch('charmhelpers.contrib.network.ip.ns_query')
    @patch('charmhelpers.contrib.network.ip.apt_install')
    def test_get_hostname_lookup_fail_gethostbyaddr_fallback(
            self, apt_install, ns_query, socket):
        fake_dns = FakeDNS('www.ubuntu.com')
        ns_query.return_value = []
        socket.return_value = ("www.ubuntu.com", "", "")
        with patch(builtin_import, side_effect=[fake_dns]):
            hn = net_ip.get_hostname('4.2.2.1')
        self.assertEquals(hn, "www.ubuntu.com")

    @patch('charmhelpers.contrib.network.ip.subprocess.call')
    def test_port_has_listener(self, subprocess_call):
        subprocess_call.return_value = 1
        self.assertEqual(net_ip.port_has_listener('ip-address', 50), False)
        subprocess_call.assert_called_with(['nc', '-z', 'ip-address', '50'])
        subprocess_call.return_value = 0
        self.assertEqual(net_ip.port_has_listener('ip-address', 70), True)
        subprocess_call.assert_called_with(['nc', '-z', 'ip-address', '70'])

    @patch.object(net_ip, 'log', lambda *args, **kwargs: None)
    @patch.object(net_ip, 'config')
    @patch.object(net_ip, 'network_get_primary_address')
    @patch.object(net_ip, 'get_address_in_network')
    @patch.object(net_ip, 'unit_get')
    @patch.object(net_ip, 'get_ipv6_addr')
    @patch.object(net_ip, 'assert_charm_supports_ipv6')
    def test_get_relation_ip(self, assert_charm_supports_ipv6, get_ipv6_addr,
                             unit_get, get_address_in_network,
                             network_get_primary_address, config):
        ACCESS_IP = '10.50.1.1'
        ACCESS_NETWORK = '10.50.1.0/24'
        AMQP_IP = '10.200.1.1'
        IPV6_IP = '2001:DB8::1'
        DEFAULT_IP = '172.16.1.1'
        assert_charm_supports_ipv6.return_value = True
        get_ipv6_addr.return_value = [IPV6_IP]
        unit_get.return_value = DEFAULT_IP
        get_address_in_network.return_value = DEFAULT_IP
        network_get_primary_address.return_value = AMQP_IP

        # Network-get calls
        _config = {'prefer-ipv6': False}
        config.side_effect = lambda key: _config.get(key)

        network_get_primary_address.side_effect = NotImplementedError
        self.assertEqual(DEFAULT_IP, net_ip.get_relation_ip('amqp'))

        network_get_primary_address.side_effect = net_ip.NoNetworkBinding
        self.assertEqual(DEFAULT_IP, net_ip.get_relation_ip('doesnotexist'))

        network_get_primary_address.side_effect = None
        self.assertEqual(AMQP_IP, net_ip.get_relation_ip('amqp'))

        self.assertFalse(get_address_in_network.called)

        # Specific CIDR network
        get_address_in_network.return_value = ACCESS_IP
        network_get_primary_address.return_value = DEFAULT_IP
        self.assertEqual(
            ACCESS_IP,
            net_ip.get_relation_ip('shared-db',
                                   cidr_network=ACCESS_NETWORK))
        get_address_in_network.assert_called_with(ACCESS_NETWORK, DEFAULT_IP)

        self.assertFalse(assert_charm_supports_ipv6.called)

        # IPv6
        _config = {'prefer-ipv6': True}
        config.side_effect = lambda key: _config.get(key)
        self.assertEqual(IPV6_IP, net_ip.get_relation_ip('amqp'))
        assert_charm_supports_ipv6.assert_called_with()



from __future__ import print_function

import mock
import os
import subprocess
import unittest

from charmhelpers.contrib.network import ufw

__author__ = 'Felipe Reyes <felipe.reyes@canonical.com>'


LSMOD_NO_IP6 = """Module                  Size  Used by
raid1                  39533  1
psmouse               106548  0
raid0                  17842  0
ahci                   34062  5
multipath              13145  0
r8169                  71471  0
libahci                32424  1 ahci
mii                    13934  1 r8169
linear                 12894  0
"""
LSMOD_IP6 = """Module                  Size  Used by
xt_hl                  12521  0
ip6_tables             27026  0
ip6t_rt                13537  0
nf_conntrack_ipv6      18894  0
nf_defrag_ipv6         34769  1 nf_conntrack_ipv6
xt_recent              18457  0
xt_LOG                 17702  0
xt_limit               12711  0
"""
DEFAULT_POLICY_OUTPUT = """Default incoming policy changed to 'deny'
(be sure to update your rules accordingly)
"""
DEFAULT_POLICY_OUTPUT_OUTGOING = """Default outgoing policy changed to 'allow'
(be sure to update your rules accordingly)
"""

UFW_STATUS_NUMBERED = """Status: active

     To                         Action      From
     --                         ------      ----
[ 1] 6641/tcp                   ALLOW IN    10.219.3.86                # charm-ovn-central
[12] 6641/tcp                   REJECT IN   Anywhere
[19] 6644/tcp (v6)              REJECT IN   Anywhere (v6)              # charm-ovn-central

"""


class TestUFW(unittest.TestCase):
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    @mock.patch('charmhelpers.contrib.network.ufw.modprobe')
    def test_enable_ok(self, modprobe, check_output, log):
        msg = 'Firewall is active and enabled on system startup\n'
        check_output.return_value = msg
        self.assertTrue(ufw.enable())

        check_output.assert_any_call(['ufw', 'enable'],
                                     universal_newlines=True,
                                     env={'LANG': 'en_US',
                                          'PATH': os.environ['PATH']})
        log.assert_any_call(msg, level='DEBUG')
        log.assert_any_call('ufw enabled', level='INFO')

    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    @mock.patch('charmhelpers.contrib.network.ufw.modprobe')
    def test_enable_fail(self, modprobe, check_output, log):
        msg = 'neneene\n'
        check_output.return_value = msg
        self.assertFalse(ufw.enable())

        check_output.assert_any_call(['ufw', 'enable'],
                                     universal_newlines=True,
                                     env={'LANG': 'en_US',
                                          'PATH': os.environ['PATH']})
        log.assert_any_call(msg, level='DEBUG')
        log.assert_any_call("ufw couldn't be enabled", level='WARN')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    def test_disable_ok(self, check_output, log, is_enabled):
        is_enabled.return_value = True
        msg = 'Firewall stopped and disabled on system startup\n'
        check_output.return_value = msg
        self.assertTrue(ufw.disable())

        check_output.assert_any_call(['ufw', 'disable'],
                                     universal_newlines=True,
                                     env={'LANG': 'en_US',
                                          'PATH': os.environ['PATH']})
        log.assert_any_call(msg, level='DEBUG')
        log.assert_any_call('ufw disabled', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    def test_disable_fail(self, check_output, log, is_enabled):
        is_enabled.return_value = True
        msg = 'neneene\n'
        check_output.return_value = msg
        self.assertFalse(ufw.disable())

        check_output.assert_any_call(['ufw', 'disable'],
                                     universal_newlines=True,
                                     env={'LANG': 'en_US',
                                          'PATH': os.environ['PATH']})
        log.assert_any_call(msg, level='DEBUG')
        log.assert_any_call("ufw couldn't be disabled", level='WARN')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    def test_modify_access_ufw_is_disabled(self, check_output, log,
                                           is_enabled):
        is_enabled.return_value = False
        ufw.modify_access('127.0.0.1')
        log.assert_any_call('ufw is disabled, skipping modify_access()',
                            level='WARN')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_modify_access_allow(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p

        ufw.modify_access('127.0.0.1')
        popen.assert_any_call(['ufw', 'allow', 'from', '127.0.0.1', 'to',
                               'any'], stdout=subprocess.PIPE)
        log.assert_any_call('ufw allow: ufw allow from 127.0.0.1 to any',
                            level='DEBUG')
        log.assert_any_call('stdout', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_modify_access_allow_set_proto(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p

        ufw.modify_access('127.0.0.1', proto='udp')
        popen.assert_any_call(['ufw', 'allow', 'from', '127.0.0.1', 'to',
                               'any', 'proto', 'udp'], stdout=subprocess.PIPE)
        log.assert_any_call(('ufw allow: ufw allow from 127.0.0.1 '
                             'to any proto udp'), level='DEBUG')
        log.assert_any_call('stdout', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_modify_access_allow_set_port(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p

        ufw.modify_access('127.0.0.1', port='80')
        popen.assert_any_call(['ufw', 'allow', 'from', '127.0.0.1', 'to',
                               'any', 'port', '80'], stdout=subprocess.PIPE)
        log.assert_any_call(('ufw allow: ufw allow from 127.0.0.1 '
                             'to any port 80'), level='DEBUG')
        log.assert_any_call('stdout', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_modify_access_allow_set_dst(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p

        ufw.modify_access('127.0.0.1', dst='127.0.0.1', port='80')
        popen.assert_any_call(['ufw', 'allow', 'from', '127.0.0.1', 'to',
                               '127.0.0.1', 'port', '80'],
                              stdout=subprocess.PIPE)
        log.assert_any_call(('ufw allow: ufw allow from 127.0.0.1 '
                             'to 127.0.0.1 port 80'), level='DEBUG')
        log.assert_any_call('stdout', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_modify_access_allow_ipv6(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p

        ufw.modify_access('::1', dst='::1', port='80')
        popen.assert_any_call(['ufw', 'allow', 'from', '::1', 'to',
                               '::1', 'port', '80'],
                              stdout=subprocess.PIPE)
        log.assert_any_call(('ufw allow: ufw allow from ::1 '
                             'to ::1 port 80'), level='DEBUG')
        log.assert_any_call('stdout', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_modify_access_with_index(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p

        ufw.modify_access('127.0.0.1', dst='127.0.0.1', port='80', index=1)
        popen.assert_any_call(['ufw', 'insert', '1', 'allow', 'from',
                               '127.0.0.1', 'to', '127.0.0.1', 'port', '80'],
                              stdout=subprocess.PIPE)
        log.assert_any_call(('ufw allow: ufw insert 1 allow from 127.0.0.1 '
                             'to 127.0.0.1 port 80'), level='DEBUG')
        log.assert_any_call('stdout', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_modify_access_prepend(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p
        ufw.modify_access('127.0.0.1', dst='127.0.0.1', port='80',
                          prepend=True)
        popen.assert_any_call(['ufw', 'prepend', 'allow', 'from', '127.0.0.1',
                               'to', '127.0.0.1', 'port', '80'],
                              stdout=subprocess.PIPE)
        log.assert_any_call(('ufw allow: ufw prepend allow from 127.0.0.1 '
                             'to 127.0.0.1 port 80'), level='DEBUG')
        log.assert_any_call('stdout', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_modify_access_comment(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p
        ufw.modify_access('127.0.0.1', dst='127.0.0.1', port='80',
                          comment='No comment')
        popen.assert_any_call(['ufw', 'allow', 'from', '127.0.0.1',
                               'to', '127.0.0.1', 'port', '80',
                               'comment', 'No comment'],
                              stdout=subprocess.PIPE)

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_modify_access_delete_index(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p
        ufw.modify_access(None, dst=None, action='delete', index=42)
        popen.assert_any_call(['ufw', '--force', 'delete', '42'],
                              stdout=subprocess.PIPE)

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_grant_access(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p

        ufw.grant_access('127.0.0.1', dst='127.0.0.1', port='80')
        popen.assert_any_call(['ufw', 'allow', 'from', '127.0.0.1', 'to',
                               '127.0.0.1', 'port', '80'],
                              stdout=subprocess.PIPE)
        log.assert_any_call(('ufw allow: ufw allow from 127.0.0.1 '
                             'to 127.0.0.1 port 80'), level='DEBUG')
        log.assert_any_call('stdout', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_grant_access_with_index(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p

        ufw.grant_access('127.0.0.1', dst='127.0.0.1', port='80', index=1)
        popen.assert_any_call(['ufw', 'insert', '1', 'allow', 'from',
                               '127.0.0.1', 'to', '127.0.0.1', 'port', '80'],
                              stdout=subprocess.PIPE)
        log.assert_any_call(('ufw allow: ufw insert 1 allow from 127.0.0.1 '
                             'to 127.0.0.1 port 80'), level='DEBUG')
        log.assert_any_call('stdout', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.Popen')
    def test_revoke_access(self, popen, log, is_enabled):
        is_enabled.return_value = True
        p = mock.Mock()
        p.configure_mock(**{'communicate.return_value': ('stdout', 'stderr'),
                            'returncode': 0})
        popen.return_value = p

        ufw.revoke_access('127.0.0.1', dst='127.0.0.1', port='80')
        popen.assert_any_call(['ufw', 'delete', 'allow', 'from', '127.0.0.1',
                               'to', '127.0.0.1', 'port', '80'],
                              stdout=subprocess.PIPE)
        log.assert_any_call(('ufw delete: ufw delete allow from 127.0.0.1 '
                             'to 127.0.0.1 port 80'), level='DEBUG')
        log.assert_any_call('stdout', level='INFO')

    @mock.patch('subprocess.check_output')
    def test_service_open(self, check_output):
        ufw.service('ssh', 'open')
        check_output.assert_any_call(['ufw', 'allow', 'ssh'],
                                     universal_newlines=True)

    @mock.patch('subprocess.check_output')
    def test_service_close(self, check_output):
        ufw.service('ssh', 'close')
        check_output.assert_any_call(['ufw', 'delete', 'allow', 'ssh'],
                                     universal_newlines=True)

    @mock.patch('subprocess.check_output')
    def test_service_unsupport_action(self, check_output):
        self.assertRaises(ufw.UFWError, ufw.service, 'ssh', 'nenene')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('os.path.isdir')
    @mock.patch('subprocess.call')
    @mock.patch('subprocess.check_output')
    def test_no_ipv6(self, check_output, call, isdir, log, is_enabled):
        check_output.return_value = ('Firewall is active and enabled '
                                     'on system startup\n')
        isdir.return_value = False
        call.return_value = 0
        is_enabled.return_value = False
        ufw.enable()

        call.assert_called_with(['sed', '-i', 's/IPV6=.*/IPV6=no/g',
                                 '/etc/default/ufw'])
        log.assert_any_call('IPv6 support in ufw disabled', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('os.path.isdir')
    @mock.patch('subprocess.call')
    @mock.patch('subprocess.check_output')
    @mock.patch('charmhelpers.contrib.network.ufw.modprobe')
    def test_no_ip6_tables(self, modprobe, check_output, call, isdir, log,
                           is_enabled):
        def c(*args, **kwargs):
            if args[0] == ['lsmod']:
                return LSMOD_NO_IP6
            elif args[0] == ['modprobe', 'ip6_tables']:
                return ""
            else:
                return 'Firewall is active and enabled on system startup\n'

        check_output.side_effect = c
        isdir.return_value = True
        call.return_value = 0

        is_enabled.return_value = False
        self.assertTrue(ufw.enable())

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('os.path.isdir')
    @mock.patch('charmhelpers.contrib.network.ufw.modprobe')
    @mock.patch('charmhelpers.contrib.network.ufw.is_module_loaded')
    def test_no_ip6_tables_fail_to_load(self, is_module_loaded,
                                        modprobe, isdir, log, is_enabled):
        is_module_loaded.return_value = False

        def c(m):
            raise subprocess.CalledProcessError(1, ['modprobe',
                                                    'ip6_tables'],
                                                "fail to load ip6_tables")

        modprobe.side_effect = c
        isdir.return_value = True
        is_enabled.return_value = False

        self.assertRaises(ufw.UFWIPv6Error, ufw.enable)

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('os.path.isdir')
    @mock.patch('charmhelpers.contrib.network.ufw.modprobe')
    @mock.patch('charmhelpers.contrib.network.ufw.is_module_loaded')
    @mock.patch('subprocess.call')
    @mock.patch('subprocess.check_output')
    def test_no_ip6_tables_fail_to_load_soft_fail(self, check_output,
                                                  call, is_module_loaded,
                                                  modprobe,
                                                  isdir, log, is_enabled):
        is_module_loaded.return_value = False

        def c(m):
            raise subprocess.CalledProcessError(1, ['modprobe',
                                                    'ip6_tables'],
                                                "fail to load ip6_tables")

        modprobe.side_effect = c
        isdir.return_value = True
        call.return_value = 0
        check_output.return_value = ("Firewall is active and enabled on "
                                     "system startup\n")
        is_enabled.return_value = False
        self.assertTrue(ufw.enable(soft_fail=True))
        call.assert_called_with(['sed', '-i', 's/IPV6=.*/IPV6=no/g',
                                 '/etc/default/ufw'])
        log.assert_any_call('IPv6 support in ufw disabled', level='INFO')

    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('os.path.isdir')
    @mock.patch('subprocess.call')
    @mock.patch('subprocess.check_output')
    def test_no_ipv6_failed_disabling_ufw(self, check_output, call, isdir,
                                          log, is_enabled):
        check_output.return_value = ('Firewall is active and enabled '
                                     'on system startup\n')
        isdir.return_value = False
        call.return_value = 1
        is_enabled.return_value = False
        self.assertRaises(ufw.UFWError, ufw.enable)

        call.assert_called_with(['sed', '-i', 's/IPV6=.*/IPV6=no/g',
                                 '/etc/default/ufw'])
        log.assert_any_call("Couldn't disable IPv6 support in ufw",
                            level="ERROR")

    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('charmhelpers.contrib.network.ufw.is_enabled')
    @mock.patch('os.path.isdir')
    @mock.patch('subprocess.check_output')
    @mock.patch('charmhelpers.contrib.network.ufw.modprobe')
    def test_with_ipv6(self, modprobe, check_output, isdir, is_enabled, log):
        def c(*args, **kwargs):
            if args[0] == ['lsmod']:
                return LSMOD_IP6
            else:
                return 'Firewall is active and enabled on system startup\n'

        check_output.side_effect = c
        is_enabled.return_value = False
        isdir.return_value = True
        ufw.enable()

    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    def test_change_default_policy(self, check_output, log):
        check_output.return_value = DEFAULT_POLICY_OUTPUT
        self.assertTrue(ufw.default_policy())
        check_output.asser_any_call(['ufw', 'default', 'deny', 'incoming'])

    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    def test_change_default_policy_allow_outgoing(self, check_output, log):
        check_output.return_value = DEFAULT_POLICY_OUTPUT_OUTGOING
        self.assertTrue(ufw.default_policy('allow', 'outgoing'))
        check_output.asser_any_call(['ufw', 'default', 'allow', 'outgoing'])

    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    def test_change_default_policy_unexpected_output(self, check_output, log):
        check_output.return_value = "asdf"
        self.assertFalse(ufw.default_policy())

    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    def test_change_default_policy_wrong_policy(self, check_output, log):
        self.assertRaises(ufw.UFWError, ufw.default_policy, 'asdf')

    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    def test_change_default_policy_wrong_direction(self, check_output, log):
        self.assertRaises(ufw.UFWError, ufw.default_policy, 'allow', 'asdf')

    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    @mock.patch('charmhelpers.contrib.network.ufw.modprobe')
    def test_reload_ok(self, modprobe, check_output, log):
        msg = 'Firewall reloaded\n'
        check_output.return_value = msg
        self.assertTrue(ufw.reload())

        check_output.assert_any_call(['ufw', 'reload'],
                                     universal_newlines=True,
                                     env={'LANG': 'en_US',
                                          'PATH': os.environ['PATH']})
        log.assert_any_call(msg, level='DEBUG')
        log.assert_any_call('ufw reloaded', level='INFO')

    @mock.patch('charmhelpers.core.hookenv.log')
    @mock.patch('subprocess.check_output')
    @mock.patch('charmhelpers.contrib.network.ufw.modprobe')
    def test_reload_fail(self, modprobe, check_output, log):
        msg = 'This did not work\n'
        check_output.return_value = msg
        self.assertFalse(ufw.reload())

        check_output.assert_any_call(['ufw', 'reload'],
                                     universal_newlines=True,
                                     env={'LANG': 'en_US',
                                          'PATH': os.environ['PATH']})
        log.assert_any_call(msg, level='DEBUG')
        log.assert_any_call("ufw couldn't be reloaded", level='WARN')

    def test_status(self):
        with mock.patch('subprocess.check_output') as check_output:
            check_output.return_value = UFW_STATUS_NUMBERED
            expect = {
                1: {'to': '6641/tcp', 'action': 'allow in',
                    'from': '10.219.3.86', 'ipv6': False,
                    'comment': 'charm-ovn-central'},
                12: {'to': '6641/tcp', 'action': 'reject in',
                     'from': 'any', 'ipv6': False,
                     'comment': ''},
                19: {'to': '6644/tcp', 'action': 'reject in',
                     'from': 'any', 'ipv6': True,
                     'comment': 'charm-ovn-central'},
            }
            n_rules = 0
            for n, r in ufw.status():
                self.assertDictEqual(r, expect[n])
                n_rules += 1
            self.assertEquals(n_rules, 3)



# Copyright 2014-2015 Canonical Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

''' Helpers for interacting with OpenvSwitch '''
import hashlib
import subprocess
import os
import six

from charmhelpers.fetch import apt_install


from charmhelpers.core.hookenv import (
    log, WARNING, INFO, DEBUG
)
from charmhelpers.core.host import (
    service
)


BRIDGE_TEMPLATE = """\
# This veth pair is required when neutron data-port is mapped to an existing linux bridge. lp:1635067

auto {linuxbridge_port}
iface {linuxbridge_port} inet manual
    pre-up ip link add name {linuxbridge_port} type veth peer name {ovsbridge_port}
    pre-up ip link set {ovsbridge_port} master {bridge}
    pre-up ip link set {ovsbridge_port} up
    up ip link set {linuxbridge_port} up
    down ip link del {linuxbridge_port}
"""

MAX_KERNEL_INTERFACE_NAME_LEN = 15


def get_bridges():
    """Return list of the bridges on the default openvswitch

    :returns: List of bridge names
    :rtype: List[str]
    :raises: subprocess.CalledProcessError if ovs-vsctl fails
    """
    cmd = ["ovs-vsctl", "list-br"]
    lines = subprocess.check_output(cmd).decode('utf-8').split("\n")
    maybe_bridges = [l.strip() for l in lines]
    return [b for b in maybe_bridges if b]


def get_bridge_ports(name):
    """Return a list the ports on a named bridge

    :param name: the name of the bridge to list
    :type name: str
    :returns: List of ports on the named bridge
    :rtype: List[str]
    :raises: subprocess.CalledProcessError if the ovs-vsctl command fails. If
        the named bridge doesn't exist, then the exception will be raised.
    """
    cmd = ["ovs-vsctl", "--", "list-ports", name]
    lines = subprocess.check_output(cmd).decode('utf-8').split("\n")
    maybe_ports = [l.strip() for l in lines]
    return [p for p in maybe_ports if p]


def get_bridges_and_ports_map():
    """Return dictionary of bridge to ports for the default openvswitch

    :returns: a mapping of bridge name to a list of ports.
    :rtype: Dict[str, List[str]]
    :raises: subprocess.CalledProcessError if any of the underlying ovs-vsctl
        command fail.
    """
    return {b: get_bridge_ports(b) for b in get_bridges()}


def _dict_to_vsctl_set(data, table, entity):
    """Helper that takes dictionary and provides ``ovs-vsctl set`` commands

    :param data: Additional data to attach to interface
        The keys in the data dictionary map directly to column names in the
        OpenvSwitch table specified as defined in DB-SCHEMA [0] referenced in
        RFC 7047 [1]

        There are some established conventions for keys in the external-ids
        column of various tables, consult the OVS Integration Guide [2] for
        more details.

        NOTE(fnordahl): Technically the ``external-ids`` column is called
        ``external_ids`` (with an underscore) and we rely on ``ovs-vsctl``'s
        behaviour of transforming dashes to underscores for us [3] so we can
        have a more pleasant data structure.

        0: http://www.openvswitch.org/ovs-vswitchd.conf.db.5.pdf
        1: https://tools.ietf.org/html/rfc7047
        2: http://docs.openvswitch.org/en/latest/topics/integration/
        3: https://github.com/openvswitch/ovs/blob/
               20dac08fdcce4b7fda1d07add3b346aa9751cfbc/
                   lib/db-ctl-base.c#L189-L215
    :type data: Optional[Dict[str,Union[str,Dict[str,str]]]]
    :param table: Name of table to operate on
    :type table: str
    :param entity: Name of entity to operate on
    :type entity: str
    :returns: '--' separated ``ovs-vsctl set`` commands
    :rtype: Iterator[Tuple[str, str, str, str, str]]
    """
    for (k, v) in data.items():
        if isinstance(v, dict):
            entries = {
                '{}:{}'.format(k, dk): dv for (dk, dv) in v.items()}
        else:
            entries = {k: v}
        for (colk, colv) in entries.items():
            yield ('--', 'set', table, entity, '{}={}'.format(colk, colv))


def add_bridge(name, datapath_type=None, brdata=None, exclusive=False):
    """Add the named bridge to openvswitch and set/update bridge data for it

    :param name: Name of bridge to create
    :type name: str
    :param datapath_type: Add datapath_type to bridge (DEPRECATED, use brdata)
    :type datapath_type: Optional[str]
    :param brdata: Additional data to attach to bridge
        The keys in the brdata dictionary map directly to column names in the
        OpenvSwitch bridge table as defined in DB-SCHEMA [0] referenced in
        RFC 7047 [1]

        There are some established conventions for keys in the external-ids
        column of various tables, consult the OVS Integration Guide [2] for
        more details.

        NOTE(fnordahl): Technically the ``external-ids`` column is called
        ``external_ids`` (with an underscore) and we rely on ``ovs-vsctl``'s
        behaviour of transforming dashes to underscores for us [3] so we can
        have a more pleasant data structure.

        0: http://www.openvswitch.org/ovs-vswitchd.conf.db.5.pdf
        1: https://tools.ietf.org/html/rfc7047
        2: http://docs.openvswitch.org/en/latest/topics/integration/
        3: https://github.com/openvswitch/ovs/blob/
               20dac08fdcce4b7fda1d07add3b346aa9751cfbc/
                   lib/db-ctl-base.c#L189-L215
    :type brdata: Optional[Dict[str,Union[str,Dict[str,str]]]]
    :param exclusive: If True, raise exception if bridge exists
    :type exclusive: bool
    :raises: subprocess.CalledProcessError
    """
    log('Creating bridge {}'.format(name))
    cmd = ['ovs-vsctl', '--']
    if not exclusive:
        cmd.append('--may-exist')
    cmd.extend(('add-br', name))
    if brdata:
        for setcmd in _dict_to_vsctl_set(brdata, 'bridge', name):
            cmd.extend(setcmd)
    if datapath_type is not None:
        log('DEPRECATION WARNING: add_bridge called with datapath_type, '
            'please use the brdata keyword argument instead.')
        cmd += ['--', 'set', 'bridge', name,
                'datapath_type={}'.format(datapath_type)]
    subprocess.check_call(cmd)


def del_bridge(name):
    """Delete the named bridge from openvswitch

    :param name: Name of bridge to remove
    :type name: str
    :raises: subprocess.CalledProcessError
    """
    log('Deleting bridge {}'.format(name))
    subprocess.check_call(["ovs-vsctl", "--", "--if-exists", "del-br", name])


def add_bridge_port(name, port, promisc=False, ifdata=None, exclusive=False,
                    linkup=True, portdata=None):
    """Add port to bridge and optionally set/update interface data for it

    :param name: Name of bridge to attach port to
    :type name: str
    :param port: Name of port as represented in netdev
    :type port: str
    :param promisc: Whether to set promiscuous mode on interface
                    True=on, False=off, None leave untouched
    :type promisc: Optional[bool]
    :param ifdata: Additional data to attach to interface
        The keys in the ifdata dictionary map directly to column names in the
        OpenvSwitch Interface table as defined in DB-SCHEMA [0] referenced in
        RFC 7047 [1]

        There are some established conventions for keys in the external-ids
        column of various tables, consult the OVS Integration Guide [2] for
        more details.

        NOTE(fnordahl): Technically the ``external-ids`` column is called
        ``external_ids`` (with an underscore) and we rely on ``ovs-vsctl``'s
        behaviour of transforming dashes to underscores for us [3] so we can
        have a more pleasant data structure.

        0: http://www.openvswitch.org/ovs-vswitchd.conf.db.5.pdf
        1: https://tools.ietf.org/html/rfc7047
        2: http://docs.openvswitch.org/en/latest/topics/integration/
        3: https://github.com/openvswitch/ovs/blob/
               20dac08fdcce4b7fda1d07add3b346aa9751cfbc/
                   lib/db-ctl-base.c#L189-L215
    :type ifdata: Optional[Dict[str,Union[str,Dict[str,str]]]]
    :param exclusive: If True, raise exception if port exists
    :type exclusive: bool
    :param linkup: Bring link up
    :type linkup: bool
    :param portdata: Additional data to attach to port. Similar to ifdata.
    :type portdata: Optional[Dict[str,Union[str,Dict[str,str]]]]
    :raises: subprocess.CalledProcessError
    """
    cmd = ['ovs-vsctl', '--']
    if not exclusive:
        cmd.append('--may-exist')
    cmd.extend(('add-port', name, port))
    for ovs_table, data in (('Interface', ifdata), ('Port', portdata)):
        if data:
            for setcmd in _dict_to_vsctl_set(data, ovs_table, port):
                cmd.extend(setcmd)

    log('Adding port {} to bridge {}'.format(port, name))
    subprocess.check_call(cmd)
    if linkup:
        # This is mostly a workaround for CI environments, in the real world
        # the bare metal provider would most likely have configured and brought
        # up the link for us.
        subprocess.check_call(["ip", "link", "set", port, "up"])
    if promisc:
        subprocess.check_call(["ip", "link", "set", port, "promisc", "on"])
    elif promisc is False:
        subprocess.check_call(["ip", "link", "set", port, "promisc", "off"])


def del_bridge_port(name, port):
    """Delete a port from the named openvswitch bridge

    :param name: Name of bridge to remove port from
    :type name: str
    :param port: Name of port to remove
    :type port: str
    :raises: subprocess.CalledProcessError
    """
    log('Deleting port {} from bridge {}'.format(port, name))
    subprocess.check_call(["ovs-vsctl", "--", "--if-exists", "del-port",
                           name, port])
    subprocess.check_call(["ip", "link", "set", port, "down"])
    subprocess.check_call(["ip", "link", "set", port, "promisc", "off"])


def add_bridge_bond(bridge, port, interfaces, portdata=None, ifdatamap=None,
                    exclusive=False):
    """Add bonded port in bridge from interfaces.

    :param bridge: Name of bridge to add bonded port to
    :type bridge: str
    :param port: Name of created port
    :type port: str
    :param interfaces: Underlying interfaces that make up the bonded port
    :type interfaces: Iterator[str]
    :param portdata: Additional data to attach to the created bond port
        See _dict_to_vsctl_set() for detailed description.
        Example:
        {
            'bond-mode': 'balance-tcp',
            'lacp': 'active',
            'other-config': {
                'lacp-time': 'fast',
            },
        }
    :type portdata: Optional[Dict[str,Union[str,Dict[str,str]]]]
    :param ifdatamap: Map of data to attach to created bond interfaces
        See _dict_to_vsctl_set() for detailed description.
        Example:
        {
            'eth0': {
                'type': 'dpdk',
                'mtu-request': '9000',
                'options': {
                    'dpdk-devargs': '0000:01:00.0',
                },
            },
        }
    :type ifdatamap: Optional[Dict[str,Dict[str,Union[str,Dict[str,str]]]]]
    :param exclusive: If True, raise exception if port exists
    :type exclusive: bool
    :raises: subprocess.CalledProcessError
    """
    cmd = ['ovs-vsctl', '--']
    if not exclusive:
        cmd.append('--may-exist')
    cmd.extend(('add-bond', bridge, port))
    cmd.extend(interfaces)
    if portdata:
        for setcmd in _dict_to_vsctl_set(portdata, 'port', port):
            cmd.extend(setcmd)
    if ifdatamap:
        for ifname, ifdata in ifdatamap.items():
            for setcmd in _dict_to_vsctl_set(ifdata, 'Interface', ifname):
                cmd.extend(setcmd)
    subprocess.check_call(cmd)


def add_ovsbridge_linuxbridge(name, bridge, ifdata=None):
    """Add linux bridge to the named openvswitch bridge

    :param name: Name of ovs bridge to be added to Linux bridge
    :type name: str
    :param bridge: Name of Linux bridge to be added to ovs bridge
    :type name: str
    :param ifdata: Additional data to attach to interface
        The keys in the ifdata dictionary map directly to column names in the
        OpenvSwitch Interface table as defined in DB-SCHEMA [0] referenced in
        RFC 7047 [1]

        There are some established conventions for keys in the external-ids
        column of various tables, consult the OVS Integration Guide [2] for
        more details.

        NOTE(fnordahl): Technically the ``external-ids`` column is called
        ``external_ids`` (with an underscore) and we rely on ``ovs-vsctl``'s
        behaviour of transforming dashes to underscores for us [3] so we can
        have a more pleasant data structure.

        0: http://www.openvswitch.org/ovs-vswitchd.conf.db.5.pdf
        1: https://tools.ietf.org/html/rfc7047
        2: http://docs.openvswitch.org/en/latest/topics/integration/
        3: https://github.com/openvswitch/ovs/blob/
               20dac08fdcce4b7fda1d07add3b346aa9751cfbc/
                   lib/db-ctl-base.c#L189-L215
    :type ifdata: Optional[Dict[str,Union[str,Dict[str,str]]]]
    """
    try:
        import netifaces
    except ImportError:
        if six.PY2:
            apt_install('python-netifaces', fatal=True)
        else:
            apt_install('python3-netifaces', fatal=True)
        import netifaces

    # NOTE(jamespage):
    # Older code supported addition of a linuxbridge directly
    # to an OVS bridge; ensure we don't break uses on upgrade
    existing_ovs_bridge = port_to_br(bridge)
    if existing_ovs_bridge is not None:
        log('Linuxbridge {} is already directly in use'
            ' by OVS bridge {}'.format(bridge, existing_ovs_bridge),
            level=INFO)
        return

    # NOTE(jamespage):
    # preserve existing naming because interfaces may already exist.
    ovsbridge_port = "veth-" + name
    linuxbridge_port = "veth-" + bridge
    if (len(ovsbridge_port) > MAX_KERNEL_INTERFACE_NAME_LEN or
            len(linuxbridge_port) > MAX_KERNEL_INTERFACE_NAME_LEN):
        # NOTE(jamespage):
        # use parts of hashed bridgename (openstack style) when
        # a bridge name exceeds 15 chars
        hashed_bridge = hashlib.sha256(bridge.encode('UTF-8')).hexdigest()
        base = '{}-{}'.format(hashed_bridge[:8], hashed_bridge[-2:])
        ovsbridge_port = "cvo{}".format(base)
        linuxbridge_port = "cvb{}".format(base)

    interfaces = netifaces.interfaces()
    for interface in interfaces:
        if interface == ovsbridge_port or interface == linuxbridge_port:
            log('Interface {} already exists'.format(interface), level=INFO)
            return

    log('Adding linuxbridge {} to ovsbridge {}'.format(bridge, name),
        level=INFO)

    check_for_eni_source()

    with open('/etc/network/interfaces.d/{}.cfg'.format(
            linuxbridge_port), 'w') as config:
        config.write(BRIDGE_TEMPLATE.format(linuxbridge_port=linuxbridge_port,
                                            ovsbridge_port=ovsbridge_port,
                                            bridge=bridge))

    subprocess.check_call(["ifup", linuxbridge_port])
    add_bridge_port(name, linuxbridge_port, ifdata=ifdata)


def is_linuxbridge_interface(port):
    ''' Check if the interface is a linuxbridge bridge
    :param port: Name of an interface to check whether it is a Linux bridge
    :returns: True if port is a Linux bridge'''

    if os.path.exists('/sys/class/net/' + port + '/bridge'):
        log('Interface {} is a Linux bridge'.format(port), level=DEBUG)
        return True
    else:
        log('Interface {} is not a Linux bridge'.format(port), level=DEBUG)
        return False


def set_manager(manager):
    ''' Set the controller for the local openvswitch '''
    log('Setting manager for local ovs to {}'.format(manager))
    subprocess.check_call(['ovs-vsctl', 'set-manager',
                           'ssl:{}'.format(manager)])


def set_Open_vSwitch_column_value(column_value):
    """
    Calls ovs-vsctl and sets the 'column_value' in the Open_vSwitch table.

    :param column_value:
            See http://www.openvswitch.org//ovs-vswitchd.conf.db.5.pdf for
            details of the relevant values.
    :type str
    :raises CalledProcessException: possibly ovsdb-server is not running
    """
    log('Setting {} in the Open_vSwitch table'.format(column_value))
    subprocess.check_call(['ovs-vsctl', 'set', 'Open_vSwitch', '.', column_value])


CERT_PATH = '/etc/openvswitch/ovsclient-cert.pem'


def get_certificate():
    ''' Read openvswitch certificate from disk '''
    if os.path.exists(CERT_PATH):
        log('Reading ovs certificate from {}'.format(CERT_PATH))
        with open(CERT_PATH, 'r') as cert:
            full_cert = cert.read()
            begin_marker = "-----BEGIN CERTIFICATE-----"
            end_marker = "-----END CERTIFICATE-----"
            begin_index = full_cert.find(begin_marker)
            end_index = full_cert.rfind(end_marker)
            if end_index == -1 or begin_index == -1:
                raise RuntimeError("Certificate does not contain valid begin"
                                   " and end markers.")
            full_cert = full_cert[begin_index:(end_index + len(end_marker))]
            return full_cert
    else:
        log('Certificate not found', level=WARNING)
        return None


def check_for_eni_source():
    ''' Juju removes the source line when setting up interfaces,
    replace if missing '''

    with open('/etc/network/interfaces', 'r') as eni:
        for line in eni:
            if line == 'source /etc/network/interfaces.d/*':
                return
    with open('/etc/network/interfaces', 'a') as eni:
        eni.write('\nsource /etc/network/interfaces.d/*')


def full_restart():
    ''' Full restart and reload of openvswitch '''
    if os.path.exists('/etc/init/openvswitch-force-reload-kmod.conf'):
        service('start', 'openvswitch-force-reload-kmod')
    else:
        service('force-reload-kmod', 'openvswitch-switch')


def enable_ipfix(bridge, target,
                 cache_active_timeout=60,
                 cache_max_flows=128,
                 sampling=64):
    '''Enable IPFIX on bridge to target.
    :param bridge: Bridge to monitor
    :param target: IPFIX remote endpoint
    :param cache_active_timeout: The maximum period in seconds for
                                 which an IPFIX flow record is cached
                                 and aggregated before being sent
    :param cache_max_flows: The maximum number of IPFIX flow records
                            that can be cached at a time
    :param sampling: The rate at which packets should be sampled and
                     sent to each target collector
    '''
    cmd = [
        'ovs-vsctl', 'set', 'Bridge', bridge, 'ipfix=@i', '--',
        '--id=@i', 'create', 'IPFIX',
        'targets="{}"'.format(target),
        'sampling={}'.format(sampling),
        'cache_active_timeout={}'.format(cache_active_timeout),
        'cache_max_flows={}'.format(cache_max_flows),
    ]
    log('Enabling IPfix on {}.'.format(bridge))
    subprocess.check_call(cmd)


def disable_ipfix(bridge):
    '''Diable IPFIX on target bridge.
    :param bridge: Bridge to modify
    '''
    cmd = ['ovs-vsctl', 'clear', 'Bridge', bridge, 'ipfix']
    subprocess.check_call(cmd)


def port_to_br(port):
    '''Determine the bridge that contains a port
    :param port: Name of port to check for
    :returns str: OVS bridge containing port or None if not found
    '''
    try:
        return subprocess.check_output(
            ['ovs-vsctl', 'port-to-br', port]
        ).decode('UTF-8').strip()
    except subprocess.CalledProcessError:
        return None


def ovs_appctl(target, args):
    """Run `ovs-appctl` for target with args and return output.

    :param target: Name of daemon to contact.  Unless target begins with '/',
                   `ovs-appctl` looks for a pidfile and will build the path to
                   a /var/run/openvswitch/target.pid.ctl for you.
    :type target: str
    :param args: Command and arguments to pass to `ovs-appctl`
    :type args: Tuple[str, ...]
    :returns: Output from command
    :rtype: str
    :raises: subprocess.CalledProcessError
    """
    cmd = ['ovs-appctl', '-t', target]
    cmd.extend(args)
    return subprocess.check_output(cmd, universal_newlines=True)



import unittest
from mock import call, patch
import yaml

import tools.charm_helpers_sync.charm_helpers_sync as sync

import six
if not six.PY3:
    builtin_open = '__builtin__.open'
else:
    builtin_open = 'builtins.open'


INCLUDE = """
include:
    - core
    - contrib.openstack
    - contrib.storage
    - contrib.hahelpers:
        - utils
        - ceph_utils
        - cluster_utils
        - haproxy_utils
"""


class HelperSyncTests(unittest.TestCase):
    def test_clone_helpers(self):
        '''It properly branches the correct helpers branch'''
        with patch('subprocess.check_call') as check_call:
            sync.clone_helpers(work_dir='/tmp/foo', repo='git:charm-helpers')
            check_call.assert_called_with(['git',
                                           'clone', '--depth=1',
                                           'git:charm-helpers',
                                           '/tmp/foo/charm-helpers'])

    def test_module_path(self):
        '''It converts a python module path to a filesystem path'''
        self.assertEquals(sync._module_path('some.test.module'),
                          'some/test/module')

    def test_src_path(self):
        '''It renders the correct path to module within charm-helpers tree'''
        path = sync._src_path(src='/tmp/charm-helpers',
                              module='contrib.openstack')
        self.assertEquals('/tmp/charm-helpers/charmhelpers/contrib/openstack',
                          path)

    def test_dest_path(self):
        '''It correctly finds the correct install path within a charm'''
        path = sync._dest_path(dest='/tmp/mycharm/hooks/charmhelpers',
                               module='contrib.openstack')
        self.assertEquals('/tmp/mycharm/hooks/charmhelpers/contrib/openstack',
                          path)

    @patch(builtin_open)
    @patch('os.path.exists')
    @patch('os.walk')
    def test_ensure_init(self, walk, exists, _open):
        '''It ensures all subdirectories of a parent are python importable'''
        # os walk
        # os.path.join
        # os.path.exists
        # open
        def _walk(path):
            yield ('/tmp/hooks/', ['helpers'], [])
            yield ('/tmp/hooks/helpers', ['foo'], [])
            yield ('/tmp/hooks/helpers/foo', [], [])
        walk.side_effect = _walk
        exists.return_value = False
        sync.ensure_init('hooks/helpers/foo/')
        ex = [call('/tmp/hooks/__init__.py', 'wb'),
              call('/tmp/hooks/helpers/__init__.py', 'wb'),
              call('/tmp/hooks/helpers/foo/__init__.py', 'wb')]
        for c in ex:
            self.assertIn(c, _open.call_args_list)

    @patch('tools.charm_helpers_sync.charm_helpers_sync.ensure_init')
    @patch('os.path.isfile')
    @patch('shutil.copy')
    @patch('os.makedirs')
    @patch('os.path.exists')
    def test_sync_pyfile(self, exists, mkdirs, copy, isfile, ensure_init):
        '''It correctly syncs a py src file from src to dest'''
        exists.return_value = False
        isfile.return_value = True
        sync.sync_pyfile('/tmp/charm-helpers/core/host',
                         'hooks/charmhelpers/core')
        mkdirs.assert_called_with('hooks/charmhelpers/core')
        copy_f = call('/tmp/charm-helpers/core/host.py',
                      'hooks/charmhelpers/core')
        copy_i = call('/tmp/charm-helpers/core/__init__.py',
                      'hooks/charmhelpers/core')
        self.assertIn(copy_f, copy.call_args_list)
        self.assertIn(copy_i, copy.call_args_list)
        ensure_init.assert_called_with('hooks/charmhelpers/core')

    def _test_filter_dir(self, opts, isfile, isdir):
        '''It filters non-python files and non-module dirs from source'''
        files = {
            'bad_file.bin': 'f',
            'some_dir': 'd',
            'good_helper.py': 'f',
            'good_helper2.py': 'f',
            'good_helper3.py': 'f',
            'bad_file.img': 'f',
        }

        def _isfile(f):
            try:
                return files[f.split('/').pop()] == 'f'
            except KeyError:
                return False

        def _isdir(f):
            try:
                return files[f.split('/').pop()] == 'd'
            except KeyError:
                return False

        isfile.side_effect = _isfile
        isdir.side_effect = _isdir
        result = sync.get_filter(opts)(dir='/tmp/charm-helpers/core',
                                       ls=six.iterkeys(files))
        return result

    @patch('os.path.isdir')
    @patch('os.path.isfile')
    def test_filter_dir_no_opts(self, isfile, isdir):
        '''It filters out all non-py files by default'''
        result = self._test_filter_dir(opts=None, isfile=isfile, isdir=isdir)
        ex = ['bad_file.bin', 'bad_file.img', 'some_dir']
        self.assertEquals(sorted(ex), sorted(result))

    @patch('os.path.isdir')
    @patch('os.path.isfile')
    def test_filter_dir_with_include(self, isfile, isdir):
        '''It includes non-py files if specified as an include opt'''
        result = sorted(self._test_filter_dir(opts=['inc=*.img'],
                                              isfile=isfile, isdir=isdir))
        ex = sorted(['bad_file.bin', 'some_dir'])
        self.assertEquals(ex, result)

    @patch('os.path.isdir')
    @patch('os.path.isfile')
    def test_filter_dir_include_all(self, isfile, isdir):
        '''It does not filter anything if option specified to include all'''
        self.assertEquals(sync.get_filter(opts=['inc=*']), None)

    @patch('tools.charm_helpers_sync.charm_helpers_sync.get_filter')
    @patch('tools.charm_helpers_sync.charm_helpers_sync.ensure_init')
    @patch('shutil.copytree')
    @patch('shutil.rmtree')
    @patch('os.path.exists')
    def test_sync_directory(self, exists, rmtree, copytree, ensure_init,
                            _filter):
        '''It correctly syncs src directory to dest directory'''
        _filter.return_value = None
        sync.sync_directory('/tmp/charm-helpers/charmhelpers/core',
                            'hooks/charmhelpers/core')
        exists.return_value = True
        rmtree.assert_called_with('hooks/charmhelpers/core')
        copytree.assert_called_with('/tmp/charm-helpers/charmhelpers/core',
                                    'hooks/charmhelpers/core', ignore=None)
        ensure_init.assert_called_with('hooks/charmhelpers/core')

    @patch('os.path.isfile')
    def test_is_pyfile(self, isfile):
        '''It correctly identifies incomplete path to a py src file as such'''
        sync._is_pyfile('/tmp/charm-helpers/charmhelpers/core/host')
        isfile.assert_called_with(
            '/tmp/charm-helpers/charmhelpers/core/host.py'
        )

    @patch('tools.charm_helpers_sync.charm_helpers_sync.sync_pyfile')
    @patch('tools.charm_helpers_sync.charm_helpers_sync.sync_directory')
    @patch('os.path.isdir')
    def test_syncs_directory(self, is_dir, sync_dir, sync_pyfile):
        '''It correctly syncs a module directory'''
        is_dir.return_value = True
        sync.sync(src='/tmp/charm-helpers',
                  dest='hooks/charmhelpers',
                  module='contrib.openstack')

        sync_dir.assert_called_with(
            '/tmp/charm-helpers/charmhelpers/contrib/openstack',
            'hooks/charmhelpers/contrib/openstack', None)

        # __init__.py files leading to the directory were also synced.
        sync_pyfile.assert_has_calls([
            call('/tmp/charm-helpers/charmhelpers/__init__',
                 'hooks/charmhelpers'),
            call('/tmp/charm-helpers/charmhelpers/contrib/__init__',
                 'hooks/charmhelpers/contrib')])

    @patch('tools.charm_helpers_sync.charm_helpers_sync.sync_pyfile')
    @patch('tools.charm_helpers_sync.charm_helpers_sync._is_pyfile')
    @patch('os.path.isdir')
    def test_syncs_file(self, is_dir, is_pyfile, sync_pyfile):
        '''It correctly syncs a module file'''
        is_dir.return_value = False
        is_pyfile.return_value = True
        sync.sync(src='/tmp/charm-helpers',
                  dest='hooks/charmhelpers',
                  module='contrib.openstack.utils')
        sync_pyfile.assert_has_calls([
            call('/tmp/charm-helpers/charmhelpers/__init__',
                 'hooks/charmhelpers'),
            call('/tmp/charm-helpers/charmhelpers/contrib/__init__',
                 'hooks/charmhelpers/contrib'),
            call('/tmp/charm-helpers/charmhelpers/contrib/openstack/__init__',
                 'hooks/charmhelpers/contrib/openstack'),
            call('/tmp/charm-helpers/charmhelpers/contrib/openstack/utils',
                 'hooks/charmhelpers/contrib/openstack')])

    @patch('tools.charm_helpers_sync.charm_helpers_sync.sync')
    @patch('os.path.isdir')
    @patch('os.path.exists')
    def test_sync_helpers_from_config(self, exists, isdir, _sync):
        '''It correctly syncs a list of included helpers'''
        include = yaml.safe_load(INCLUDE)['include']
        isdir.return_value = True
        exists.return_value = False
        sync.sync_helpers(include=include,
                          src='/tmp/charm-helpers',

                          dest='hooks/charmhelpers')
        mods = [
            'core',
            'contrib.openstack',
            'contrib.storage',
            'contrib.hahelpers.utils',
            'contrib.hahelpers.ceph_utils',
            'contrib.hahelpers.cluster_utils',
            'contrib.hahelpers.haproxy_utils'
        ]

        ex_calls = []
        [ex_calls.append(
            call('/tmp/charm-helpers', 'hooks/charmhelpers', c, [])
        ) for c in mods]
        self.assertEquals(ex_calls, _sync.call_args_list)

    @patch('tools.charm_helpers_sync.charm_helpers_sync.sync')
    @patch('os.path.isdir')
    @patch('os.path.exists')
    @patch('shutil.rmtree')
    def test_sync_helpers_from_config_cleanup(self, _rmtree, _exists,
                                              isdir, _sync):
        '''It correctly syncs a list of included helpers'''
        include = yaml.safe_load(INCLUDE)['include']
        isdir.return_value = True
        _exists.return_value = True

        sync.sync_helpers(include=include,
                          src='/tmp/charm-helpers',

                          dest='hooks/charmhelpers')
        _rmtree.assert_called_with('hooks/charmhelpers')
        mods = [
            'core',
            'contrib.openstack',
            'contrib.storage',
            'contrib.hahelpers.utils',
            'contrib.hahelpers.ceph_utils',
            'contrib.hahelpers.cluster_utils',
            'contrib.hahelpers.haproxy_utils'
        ]

        ex_calls = []
        [ex_calls.append(
            call('/tmp/charm-helpers', 'hooks/charmhelpers', c, [])
        ) for c in mods]
        self.assertEquals(ex_calls, _sync.call_args_list)

    def test_extract_option_no_globals(self):
        '''It extracts option from an included item with no global options'''
        inc = 'contrib.openstack.templates|inc=*.template'
        result = sync.extract_options(inc)
        ex = ('contrib.openstack.templates', ['inc=*.template'])
        self.assertEquals(ex, result)

    def test_extract_option_with_global_as_string(self):
        '''It extracts option for include with global options as str'''
        inc = 'contrib.openstack.templates|inc=*.template'
        result = sync.extract_options(inc, global_options='inc=foo.*')
        ex = ('contrib.openstack.templates',
              ['inc=*.template', 'inc=foo.*'])
        self.assertEquals(ex, result)

    def test_extract_option_with_globals(self):
        '''It extracts option from an included item with global options'''
        inc = 'contrib.openstack.templates|inc=*.template'
        result = sync.extract_options(inc, global_options=['inc=*.cfg'])
        ex = ('contrib.openstack.templates', ['inc=*.template', 'inc=*.cfg'])
        self.assertEquals(ex, result)

    def test_extract_multiple_options_with_globals(self):
        '''It extracts multiple options from an included item'''
        inc = 'contrib.openstack.templates|inc=*.template,inc=foo.*'
        result = sync.extract_options(inc, global_options=['inc=*.cfg'])
        ex = ('contrib.openstack.templates',
              ['inc=*.template', 'inc=foo.*', 'inc=*.cfg'])
        self.assertEquals(ex, result)



import collections
import json
import mock
import six
import unittest

from charmhelpers.contrib.openstack import utils

if not six.PY3:
    builtin_open = '__builtin__.open'
else:
    builtin_open = 'builtins.open'


class UtilsTests(unittest.TestCase):
    def setUp(self):
        super(UtilsTests, self).setUp()

    def test_compare_openstack_comparator(self):
        self.assertTrue(utils.CompareOpenStackReleases('mitaka') < 'newton')
        self.assertTrue(utils.CompareOpenStackReleases('pike') > 'essex')

    @mock.patch.object(utils, 'config')
    @mock.patch('charmhelpers.contrib.openstack.utils.relation_set')
    @mock.patch('charmhelpers.contrib.openstack.utils.relation_ids')
    @mock.patch('charmhelpers.contrib.openstack.utils.get_ipv6_addr')
    def test_sync_db_with_multi_ipv6_addresses(self, mock_get_ipv6_addr,
                                               mock_relation_ids,
                                               mock_relation_set,
                                               mock_config):
        mock_config.return_value = None
        addr1 = '2001:db8:1:0:f816:3eff:fe45:7c/64'
        addr2 = '2001:db8:1:0:d0cf:528c:23eb:5000/64'
        mock_get_ipv6_addr.return_value = [addr1, addr2]
        mock_relation_ids.return_value = ['shared-db']

        utils.sync_db_with_multi_ipv6_addresses('testdb', 'testdbuser')
        hosts = json.dumps([addr1, addr2])
        mock_relation_set.assert_called_with(relation_id='shared-db',
                                             database='testdb',
                                             username='testdbuser',
                                             hostname=hosts)

    @mock.patch.object(utils, 'config')
    @mock.patch('charmhelpers.contrib.openstack.utils.relation_set')
    @mock.patch('charmhelpers.contrib.openstack.utils.relation_ids')
    @mock.patch('charmhelpers.contrib.openstack.utils.get_ipv6_addr')
    def test_sync_db_with_multi_ipv6_addresses_single(self, mock_get_ipv6_addr,
                                                      mock_relation_ids,
                                                      mock_relation_set,
                                                      mock_config):
        mock_config.return_value = None
        addr1 = '2001:db8:1:0:f816:3eff:fe45:7c/64'
        mock_get_ipv6_addr.return_value = [addr1]
        mock_relation_ids.return_value = ['shared-db']

        utils.sync_db_with_multi_ipv6_addresses('testdb', 'testdbuser')
        hosts = json.dumps([addr1])
        mock_relation_set.assert_called_with(relation_id='shared-db',
                                             database='testdb',
                                             username='testdbuser',
                                             hostname=hosts)

    @mock.patch.object(utils, 'config')
    @mock.patch('charmhelpers.contrib.openstack.utils.relation_set')
    @mock.patch('charmhelpers.contrib.openstack.utils.relation_ids')
    @mock.patch('charmhelpers.contrib.openstack.utils.get_ipv6_addr')
    def test_sync_db_with_multi_ipv6_addresses_w_prefix(self,
                                                        mock_get_ipv6_addr,
                                                        mock_relation_ids,
                                                        mock_relation_set,
                                                        mock_config):
        mock_config.return_value = None
        addr1 = '2001:db8:1:0:f816:3eff:fe45:7c/64'
        mock_get_ipv6_addr.return_value = [addr1]
        mock_relation_ids.return_value = ['shared-db']

        utils.sync_db_with_multi_ipv6_addresses('testdb', 'testdbuser',
                                                relation_prefix='bungabunga')
        hosts = json.dumps([addr1])
        mock_relation_set.assert_called_with(relation_id='shared-db',
                                             bungabunga_database='testdb',
                                             bungabunga_username='testdbuser',
                                             bungabunga_hostname=hosts)

    @mock.patch.object(utils, 'config')
    @mock.patch('charmhelpers.contrib.openstack.utils.relation_set')
    @mock.patch('charmhelpers.contrib.openstack.utils.relation_ids')
    @mock.patch('charmhelpers.contrib.openstack.utils.get_ipv6_addr')
    def test_sync_db_with_multi_ipv6_addresses_vips(self, mock_get_ipv6_addr,
                                                    mock_relation_ids,
                                                    mock_relation_set,
                                                    mock_config):
        addr1 = '2001:db8:1:0:f816:3eff:fe45:7c/64'
        addr2 = '2001:db8:1:0:d0cf:528c:23eb:5000/64'
        vip1 = '2001:db8:1:0:f816:3eff:32b3:7c'
        vip2 = '2001:db8:1:0:f816:3eff:32b3:7d'
        mock_config.return_value = '%s 10.0.0.1 %s' % (vip1, vip2)

        mock_get_ipv6_addr.return_value = [addr1, addr2]
        mock_relation_ids.return_value = ['shared-db']

        utils.sync_db_with_multi_ipv6_addresses('testdb', 'testdbuser')
        hosts = json.dumps([addr1, addr2, vip1, vip2])
        mock_relation_set.assert_called_with(relation_id='shared-db',
                                             database='testdb',
                                             username='testdbuser',
                                             hostname=hosts)

    @mock.patch('uuid.uuid4')
    @mock.patch('charmhelpers.contrib.openstack.utils.related_units')
    @mock.patch('charmhelpers.contrib.openstack.utils.relation_set')
    @mock.patch('charmhelpers.contrib.openstack.utils.relation_ids')
    def test_remote_restart(self, mock_relation_ids, mock_relation_set,
                            mock_related_units, mock_uuid4):
        mock_relation_ids.return_value = ['neutron-plugin-api-subordinate:8']
        mock_related_units.return_value = ['neutron-api/0']
        mock_uuid4.return_value = 'uuid4'
        utils.remote_restart('neutron-plugin-api-subordinate')
        mock_relation_set.assert_called_with(
            relation_id='neutron-plugin-api-subordinate:8',
            relation_settings={'restart-trigger': 'uuid4'}
        )

    @mock.patch.object(utils, 'lsb_release')
    @mock.patch.object(utils, 'config')
    @mock.patch('charmhelpers.contrib.openstack.utils.get_os_codename_package')
    @mock.patch('charmhelpers.contrib.openstack.utils.'
                'get_os_codename_install_source')
    def test_os_release(self, mock_get_os_codename_install_source,
                        mock_get_os_codename_package,
                        mock_config, mock_lsb_release):
        # Wipe the modules cached os_rel
        utils._os_rel = None
        mock_lsb_release.return_value = {"DISTRIB_CODENAME": "trusty"}
        mock_get_os_codename_install_source.return_value = None
        mock_get_os_codename_package.return_value = None
        mock_config.return_value = 'cloud-pocket'
        self.assertEqual(utils.os_release('my-pkg'), 'icehouse')
        mock_get_os_codename_install_source.assert_called_once_with(
            'cloud-pocket')
        mock_get_os_codename_package.assert_called_once_with(
            'my-pkg', fatal=False)
        mock_config.assert_called_once_with('openstack-origin')
        # Next call to os_release should pickup cached version
        mock_get_os_codename_install_source.reset_mock()
        mock_get_os_codename_package.reset_mock()
        self.assertEqual(utils.os_release('my-pkg'), 'icehouse')
        self.assertFalse(mock_get_os_codename_install_source.called)
        self.assertFalse(mock_get_os_codename_package.called)
        # Call os_release and bypass cache
        mock_lsb_release.return_value = {"DISTRIB_CODENAME": "xenial"}
        mock_get_os_codename_install_source.reset_mock()
        mock_get_os_codename_package.reset_mock()
        self.assertEqual(utils.os_release('my-pkg', reset_cache=True),
                         'mitaka')
        mock_get_os_codename_install_source.assert_called_once_with(
            'cloud-pocket')
        mock_get_os_codename_package.assert_called_once_with(
            'my-pkg', fatal=False)
        # Override base
        mock_lsb_release.return_value = {"DISTRIB_CODENAME": "xenial"}
        mock_get_os_codename_install_source.reset_mock()
        mock_get_os_codename_package.reset_mock()
        self.assertEqual(utils.os_release('my-pkg', reset_cache=True, base="ocata"),
                         'ocata')
        mock_get_os_codename_install_source.assert_called_once_with(
            'cloud-pocket')
        mock_get_os_codename_package.assert_called_once_with(
            'my-pkg', fatal=False)
        # Override source key
        mock_config.reset_mock()
        mock_get_os_codename_install_source.reset_mock()
        mock_get_os_codename_package.reset_mock()
        mock_get_os_codename_package.return_value = None
        utils.os_release('my-pkg', reset_cache=True, source_key='source')
        mock_config.assert_called_once_with('source')
        mock_get_os_codename_install_source.assert_called_once_with(
            'cloud-pocket')

    @mock.patch.object(utils, 'os_release')
    @mock.patch.object(utils, 'get_os_codename_install_source')
    def test_enable_memcache(self, _get_os_codename_install_source,
                             _os_release):
        # Check call with 'release'
        self.assertFalse(utils.enable_memcache(release='icehouse'))
        self.assertTrue(utils.enable_memcache(release='ocata'))
        # Check call with 'source'
        _os_release.return_value = None
        _get_os_codename_install_source.return_value = 'icehouse'
        self.assertFalse(utils.enable_memcache(source='distro'))
        _os_release.return_value = None
        _get_os_codename_install_source.return_value = 'ocata'
        self.assertTrue(utils.enable_memcache(source='distro'))
        # Check call with 'package'
        _os_release.return_value = 'icehouse'
        _get_os_codename_install_source.return_value = None
        self.assertFalse(utils.enable_memcache(package='pkg1'))
        _os_release.return_value = 'ocata'
        _get_os_codename_install_source.return_value = None
        self.assertTrue(utils.enable_memcache(package='pkg1'))

    @mock.patch.object(utils, 'enable_memcache')
    def test_enable_token_cache_pkgs(self, _enable_memcache):
        _enable_memcache.return_value = False
        self.assertEqual(utils.token_cache_pkgs(source='distro'), [])
        _enable_memcache.return_value = True
        self.assertEqual(utils.token_cache_pkgs(source='distro'),
                         ['memcached', 'python-memcache'])

    def test_update_json_file(self):
        TEST_POLICY = """{
        "delete_image_location": "",
        "get_image_location": "",
        "set_image_location": "",
        "extra_property": "False"
        }"""

        TEST_POLICY_FILE = "/etc/glance/policy.json"

        items_to_update = {
            "get_image_location": "role:admin",
            "extra_policy": "extra",
        }

        mock_open = mock.mock_open(read_data=TEST_POLICY)
        with mock.patch(builtin_open, mock_open) as mock_file:
            utils.update_json_file(TEST_POLICY_FILE, {})
            self.assertFalse(mock_file.called)

            utils.update_json_file(TEST_POLICY_FILE, items_to_update)
            mock_file.assert_has_calls([
                mock.call(TEST_POLICY_FILE),
                mock.call(TEST_POLICY_FILE, 'w'),
            ], any_order=True)

        modified_policy = json.loads(TEST_POLICY)
        modified_policy.update(items_to_update)
        mock_open().write.assert_called_with(
            json.dumps(modified_policy, indent=4, sort_keys=True))

        tmp = json.loads(TEST_POLICY)
        tmp.update(items_to_update)
        TEST_POLICY = json.dumps(tmp)
        mock_open = mock.mock_open(read_data=TEST_POLICY)
        with mock.patch(builtin_open, mock_open) as mock_file:
            utils.update_json_file(TEST_POLICY_FILE, items_to_update)
            mock_file.assert_has_calls([
                mock.call(TEST_POLICY_FILE),
            ], any_order=True)

    def test_ordered(self):
        data = {'one': 1, 'two': 2, 'three': 3}
        expected = [('one', 1), ('three', 3), ('two', 2)]
        self.assertSequenceEqual(expected,
                                 [x for x in utils.ordered(data).items()])

        data = {
            'one': 1,
            'two': 2,
            'three': {
                'uno': 1,
                'dos': 2,
                'tres': 3
            }
        }
        expected = collections.OrderedDict()
        expected['one'] = 1
        nested = collections.OrderedDict()
        nested['dos'] = 2
        nested['tres'] = 3
        nested['uno'] = 1
        expected['three'] = nested
        expected['two'] = 2
        self.assertEqual(expected, utils.ordered(data))

        self.assertRaises(ValueError, utils.ordered, "foo")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2014-2015 Canonical Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

__author__ = "Jorge Niedbalski <jorge.niedbalski@canonical.com>"

import six

from charmhelpers.fetch import (
    apt_install,
    apt_update,
)

from charmhelpers.core.hookenv import (
    log,
    INFO,
)

try:
    from netifaces import interfaces as network_interfaces
except ImportError:
    if six.PY2:
        apt_install('python-netifaces')
    else:
        apt_install('python3-netifaces')
    from netifaces import interfaces as network_interfaces

import os
import re
import subprocess

from charmhelpers.core.kernel import modprobe

REQUIRED_MODULES = (
    "mlx4_ib",
    "mlx4_en",
    "mlx4_core",
    "ib_ipath",
    "ib_mthca",
    "ib_srpt",
    "ib_srp",
    "ib_ucm",
    "ib_isert",
    "ib_iser",
    "ib_ipoib",
    "ib_cm",
    "ib_uverbs"
    "ib_umad",
    "ib_sa",
    "ib_mad",
    "ib_core",
    "ib_addr",
    "rdma_ucm",
)

REQUIRED_PACKAGES = (
    "ibutils",
    "infiniband-diags",
    "ibverbs-utils",
)

IPOIB_DRIVERS = (
    "ib_ipoib",
)

ABI_VERSION_FILE = "/sys/class/infiniband_mad/abi_version"


class DeviceInfo(object):
    pass


def install_packages():
    apt_update()
    apt_install(REQUIRED_PACKAGES, fatal=True)


def load_modules():
    for module in REQUIRED_MODULES:
        modprobe(module, persist=True)


def is_enabled():
    """Check if infiniband is loaded on the system"""
    return os.path.exists(ABI_VERSION_FILE)


def stat():
    """Return full output of ibstat"""
    return subprocess.check_output(["ibstat"])


def devices():
    """Returns a list of IB enabled devices"""
    return subprocess.check_output(['ibstat', '-l']).splitlines()


def device_info(device):
    """Returns a DeviceInfo object with the current device settings"""

    status = subprocess.check_output([
        'ibstat', device, '-s']).splitlines()

    regexes = {
        "CA type: (.*)": "device_type",
        "Number of ports: (.*)": "num_ports",
        "Firmware version: (.*)": "fw_ver",
        "Hardware version: (.*)": "hw_ver",
        "Node GUID: (.*)": "node_guid",
        "System image GUID: (.*)": "sys_guid",
    }

    device = DeviceInfo()

    for line in status:
        for expression, key in regexes.items():
            matches = re.search(expression, line)
            if matches:
                setattr(device, key, matches.group(1))

    return device


def ipoib_interfaces():
    """Return a list of IPOIB capable ethernet interfaces"""
    interfaces = []

    for interface in network_interfaces():
        try:
            driver = re.search('^driver: (.+)$', subprocess.check_output([
                'ethtool', '-i',
                interface]), re.M).group(1)

            if driver in IPOIB_DRIVERS:
                interfaces.append(interface)
        except Exception:
            log("Skipping interface %s" % interface, level=INFO)
            continue

    return interfaces



import unittest
from mock import patch
from nose.tools import raises
import charmhelpers.contrib.openstack.neutron as neutron

TO_PATCH = [
    'log',
    'config',
    'os_release',
    'check_output',
]


class NeutronTests(unittest.TestCase):
    def setUp(self):
        for m in TO_PATCH:
            setattr(self, m, self._patch(m))

    def _patch(self, method):
        _m = patch('charmhelpers.contrib.openstack.neutron.' + method)
        mock = _m.start()
        self.addCleanup(_m.stop)
        return mock

    def test_headers_package(self):
        self.check_output.return_value = b'3.13.0-19-generic'
        kname = neutron.headers_package()
        self.assertEquals(kname, 'linux-headers-3.13.0-19-generic')

    def test_kernel_version(self):
        self.check_output.return_value = b'3.13.0-19-generic'
        kver_maj, kver_min = neutron.kernel_version()
        self.assertEquals((kver_maj, kver_min), (3, 13))

    @patch.object(neutron, 'kernel_version')
    def test_determine_dkms_package_old_kernel(self, _kernel_version):
        self.check_output.return_value = b'3.4.0-19-generic'
        _kernel_version.return_value = (3, 10)
        dkms_package = neutron.determine_dkms_package()
        self.assertEquals(dkms_package, ['linux-headers-3.4.0-19-generic',
                                         'openvswitch-datapath-dkms'])

    @patch.object(neutron, 'kernel_version')
    def test_determine_dkms_package_new_kernel(self, _kernel_version):
        _kernel_version.return_value = (3, 13)
        dkms_package = neutron.determine_dkms_package()
        self.assertEquals(dkms_package, [])

    def test_quantum_plugins(self):
        self.config.return_value = 'foo'
        plugins = neutron.quantum_plugins()
        self.assertEquals(plugins['ovs']['services'],
                          ['quantum-plugin-openvswitch-agent'])
        self.assertEquals(plugins['nvp']['services'], [])

    def test_neutron_plugins_preicehouse(self):
        self.config.return_value = 'foo'
        self.os_release.return_value = 'havana'
        plugins = neutron.neutron_plugins()
        self.assertEquals(plugins['ovs']['config'],
                          '/etc/neutron/plugins/openvswitch/ovs_neutron_plugin.ini')
        self.assertEquals(plugins['nvp']['services'], [])

    def test_neutron_plugins(self):
        self.config.return_value = 'foo'
        self.os_release.return_value = 'icehouse'
        plugins = neutron.neutron_plugins()
        self.assertEquals(plugins['ovs']['config'],
                          '/etc/neutron/plugins/ml2/ml2_conf.ini')
        self.assertEquals(plugins['nvp']['config'],
                          '/etc/neutron/plugins/vmware/nsx.ini')
        self.assertTrue('neutron-plugin-vmware' in
                        plugins['nvp']['server_packages'])
        self.assertEquals(plugins['n1kv']['config'],
                          '/etc/neutron/plugins/cisco/cisco_plugins.ini')
        self.assertEquals(plugins['Calico']['config'],
                          '/etc/neutron/plugins/ml2/ml2_conf.ini')
        self.assertEquals(plugins['plumgrid']['config'],
                          '/etc/neutron/plugins/plumgrid/plumgrid.ini')
        self.assertEquals(plugins['midonet']['config'],
                          '/etc/neutron/plugins/midonet/midonet.ini')

        self.assertEquals(plugins['nvp']['services'], [])
        self.assertEquals(plugins['nsx'], plugins['nvp'])

        self.os_release.return_value = 'kilo'
        plugins = neutron.neutron_plugins()
        self.assertEquals(plugins['midonet']['driver'],
                          'neutron.plugins.midonet.plugin.MidonetPluginV2')
        self.assertEquals(plugins['nsx']['config'],
                          '/etc/neutron/plugins/vmware/nsx.ini')

        self.os_release.return_value = 'liberty'
        self.config.return_value = 'mem-1.9'
        plugins = neutron.neutron_plugins()
        self.assertEquals(plugins['midonet']['driver'],
                          'midonet.neutron.plugin_v1.MidonetPluginV2')
        self.assertTrue('python-networking-midonet' in
                        plugins['midonet']['server_packages'])

        self.os_release.return_value = 'mitaka'
        self.config.return_value = 'mem-1.9'
        plugins = neutron.neutron_plugins()
        self.assertEquals(plugins['nsx']['config'],
                          '/etc/neutron/nsx.ini')
        self.assertTrue('python-vmware-nsx' in
                        plugins['nsx']['server_packages'])

    @patch.object(neutron, 'network_manager')
    def test_neutron_plugin_attribute_quantum(self, _network_manager):
        self.config.return_value = 'foo'
        _network_manager.return_value = 'quantum'
        plugins = neutron.neutron_plugin_attribute('ovs', 'services')
        self.assertEquals(plugins, ['quantum-plugin-openvswitch-agent'])

    @patch.object(neutron, 'network_manager')
    def test_neutron_plugin_attribute_neutron(self, _network_manager):
        self.config.return_value = 'foo'
        self.os_release.return_value = 'icehouse'
        _network_manager.return_value = 'neutron'
        plugins = neutron.neutron_plugin_attribute('ovs', 'services')
        self.assertEquals(plugins, ['neutron-plugin-openvswitch-agent'])

    @raises(Exception)
    @patch.object(neutron, 'network_manager')
    def test_neutron_plugin_attribute_foo(self, _network_manager):
        _network_manager.return_value = 'foo'
        self.assertRaises(Exception, neutron.neutron_plugin_attribute('ovs', 'services'))

    @raises(Exception)
    @patch.object(neutron, 'network_manager')
    def test_neutron_plugin_attribute_plugin_keyerror(self, _network_manager):
        self.config.return_value = 'foo'
        _network_manager.return_value = 'quantum'
        self.assertRaises(Exception, neutron.neutron_plugin_attribute('foo', 'foo'))

    @patch.object(neutron, 'network_manager')
    def test_neutron_plugin_attribute_attr_keyerror(self, _network_manager):
        self.config.return_value = 'foo'
        _network_manager.return_value = 'quantum'
        plugins = neutron.neutron_plugin_attribute('ovs', 'foo')
        self.assertEquals(plugins, None)

    @raises(Exception)
    def test_network_manager_essex(self):
        essex_cases = {
            'quantum': 'quantum',
            'neutron': 'quantum',
            'newhotness': 'newhotness',
        }
        self.os_release.return_value = 'essex'
        for nwmanager in essex_cases:
            self.config.return_value = nwmanager
            self.assertRaises(Exception, neutron.network_manager())

    def test_network_manager_folsom(self):
        folsom_cases = {
            'quantum': 'quantum',
            'neutron': 'quantum',
            'newhotness': 'newhotness',
        }
        self.os_release.return_value = 'folsom'
        for nwmanager in folsom_cases:
            self.config.return_value = nwmanager
            renamed_manager = neutron.network_manager()
            self.assertEquals(renamed_manager, folsom_cases[nwmanager])

    def test_network_manager_grizzly(self):
        grizzly_cases = {
            'quantum': 'quantum',
            'neutron': 'quantum',
            'newhotness': 'newhotness',
        }
        self.os_release.return_value = 'grizzly'
        for nwmanager in grizzly_cases:
            self.config.return_value = nwmanager
            renamed_manager = neutron.network_manager()
            self.assertEquals(renamed_manager, grizzly_cases[nwmanager])

    def test_network_manager_havana(self):
        havana_cases = {
            'quantum': 'neutron',
            'neutron': 'neutron',
            'newhotness': 'newhotness',
        }
        self.os_release.return_value = 'havana'
        for nwmanager in havana_cases:
            self.config.return_value = nwmanager
            renamed_manager = neutron.network_manager()
            self.assertEquals(renamed_manager, havana_cases[nwmanager])

    def test_network_manager_icehouse(self):
        icehouse_cases = {
            'quantum': 'neutron',
            'neutron': 'neutron',
            'newhotness': 'newhotness',
        }
        self.os_release.return_value = 'icehouse'
        for nwmanager in icehouse_cases:
            self.config.return_value = nwmanager
            renamed_manager = neutron.network_manager()
            self.assertEquals(renamed_manager, icehouse_cases[nwmanager])

    def test_parse_bridge_mappings(self):
        ret = neutron.parse_bridge_mappings(None)
        self.assertEqual(ret, {})
        ret = neutron.parse_bridge_mappings("physnet1:br0")
        self.assertEqual(ret, {'physnet1': 'br0'})
        ret = neutron.parse_bridge_mappings("physnet1:br0 physnet2:br1")
        self.assertEqual(ret, {'physnet1': 'br0', 'physnet2': 'br1'})

    def test_parse_data_port_mappings(self):
        ret = neutron.parse_data_port_mappings(None)
        self.assertEqual(ret, {})
        ret = neutron.parse_data_port_mappings('br0:eth0')
        self.assertEqual(ret, {'eth0': 'br0'})
        # Back-compat test
        ret = neutron.parse_data_port_mappings('eth0', default_bridge='br0')
        self.assertEqual(ret, {'eth0': 'br0'})
        # Multiple mappings
        ret = neutron.parse_data_port_mappings('br0:eth0 br1:eth1')
        self.assertEqual(ret, {'eth0': 'br0', 'eth1': 'br1'})
        # MultMAC mappings
        ret = neutron.parse_data_port_mappings('br0:cb:23:ae:72:f2:33 '
                                               'br0:fa:16:3e:12:97:8e')
        self.assertEqual(ret, {'cb:23:ae:72:f2:33': 'br0',
                               'fa:16:3e:12:97:8e': 'br0'})

    def test_parse_vlan_range_mappings(self):
        ret = neutron.parse_vlan_range_mappings(None)
        self.assertEqual(ret, {})
        ret = neutron.parse_vlan_range_mappings('physnet1:1001:2000')
        self.assertEqual(ret, {'physnet1': ('1001', '2000')})
        ret = neutron.parse_vlan_range_mappings('physnet1:1001:2000 physnet2:2001:3000')
        self.assertEqual(ret, {'physnet1': ('1001', '2000'),
                               'physnet2': ('2001', '3000')})
        ret = neutron.parse_vlan_range_mappings('physnet1 physnet2:2001:3000')
        self.assertEqual(ret, {'physnet1': ('',),
                               'physnet2': ('2001', '3000')})



import pkg_resources
import shutil
import tempfile
import unittest
import jinja2
import os.path
import pwd
import grp

import mock
from charmhelpers.core import templating


TEMPLATES_DIR = pkg_resources.resource_filename(__name__, 'templates')


class TestTemplating(unittest.TestCase):
    def setUp(self):
        self.charm_dir = pkg_resources.resource_filename(__name__, '')
        self._charm_dir_patch = mock.patch.object(templating.hookenv,
                                                  'charm_dir')
        self._charm_dir_mock = self._charm_dir_patch.start()
        self._charm_dir_mock.side_effect = lambda: self.charm_dir

    def tearDown(self):
        self._charm_dir_patch.stop()

    @mock.patch.object(templating.host.os, 'fchown')
    @mock.patch.object(templating.host, 'mkdir')
    @mock.patch.object(templating.host, 'log')
    def test_render(self, log, mkdir, fchown):
        with tempfile.NamedTemporaryFile() as fn1, \
                tempfile.NamedTemporaryFile() as fn2:
            context = {
                'nats': {
                    'port': '1234',
                    'host': 'example.com',
                },
                'router': {
                    'domain': 'api.foo.com'
                },
                'nginx_port': 80,
            }
            templating.render('fake_cc.yml', fn1.name,
                              context, templates_dir=TEMPLATES_DIR)
            contents = open(fn1.name).read()
            self.assertRegexpMatches(contents, 'port: 1234')
            self.assertRegexpMatches(contents, 'host: example.com')
            self.assertRegexpMatches(contents, 'domain: api.foo.com')

            templating.render('test.conf', fn2.name, context,
                              templates_dir=TEMPLATES_DIR)
            contents = open(fn2.name).read()
            self.assertRegexpMatches(contents, 'listen 80')
            self.assertEqual(fchown.call_count, 2)
            # Not called, because the target directory exists. Calling
            # it would make the target directory world readable and
            # expose your secrets (!).
            self.assertEqual(mkdir.call_count, 0)

    @mock.patch.object(templating.host.os, 'fchown')
    @mock.patch.object(templating.host, 'mkdir')
    @mock.patch.object(templating.host, 'log')
    def test_render_from_string(self, log, mkdir, fchown):
        with tempfile.NamedTemporaryFile() as fn:
            context = {
                'foo': 'bar'
            }

            config_template = '{{ foo }}'
            templating.render('somefile.txt', fn.name,
                              context, templates_dir=TEMPLATES_DIR,
                              config_template=config_template)
            contents = open(fn.name).read()
            self.assertRegexpMatches(contents, 'bar')

            self.assertEqual(fchown.call_count, 1)
            # Not called, because the target directory exists. Calling
            # it would make the target directory world readable and
            # expose your secrets (!).
            self.assertEqual(mkdir.call_count, 0)

    @mock.patch.object(templating.host.os, 'fchown')
    @mock.patch.object(templating.host, 'mkdir')
    @mock.patch.object(templating.host, 'log')
    def test_render_loader(self, log, mkdir, fchown):
        with tempfile.NamedTemporaryFile() as fn1:
            context = {
                'nats': {
                    'port': '1234',
                    'host': 'example.com',
                },
                'router': {
                    'domain': 'api.foo.com'
                },
                'nginx_port': 80,
            }
            template_loader = jinja2.ChoiceLoader([jinja2.FileSystemLoader(TEMPLATES_DIR)])
            templating.render('fake_cc.yml', fn1.name,
                              context, template_loader=template_loader)
            contents = open(fn1.name).read()
            self.assertRegexpMatches(contents, 'port: 1234')
            self.assertRegexpMatches(contents, 'host: example.com')
            self.assertRegexpMatches(contents, 'domain: api.foo.com')

    @mock.patch.object(templating.os.path, 'exists')
    @mock.patch.object(templating.host.os, 'fchown')
    @mock.patch.object(templating.host, 'mkdir')
    @mock.patch.object(templating.host, 'log')
    def test_render_no_dir(self, log, mkdir, fchown, exists):
        exists.return_value = False
        with tempfile.NamedTemporaryFile() as fn1, \
                tempfile.NamedTemporaryFile() as fn2:
            context = {
                'nats': {
                    'port': '1234',
                    'host': 'example.com',
                },
                'router': {
                    'domain': 'api.foo.com'
                },
                'nginx_port': 80,
            }
            templating.render('fake_cc.yml', fn1.name,
                              context, templates_dir=TEMPLATES_DIR)
            contents = open(fn1.name).read()
            self.assertRegexpMatches(contents, 'port: 1234')
            self.assertRegexpMatches(contents, 'host: example.com')
            self.assertRegexpMatches(contents, 'domain: api.foo.com')

            templating.render('test.conf', fn2.name, context,
                              templates_dir=TEMPLATES_DIR)
            contents = open(fn2.name).read()
            self.assertRegexpMatches(contents, 'listen 80')
            self.assertEqual(fchown.call_count, 2)
            # Target directory was created, world readable (!).
            self.assertEqual(mkdir.call_count, 2)

    @mock.patch.object(templating.host.os, 'fchown')
    @mock.patch.object(templating.host, 'log')
    def test_render_2(self, log, fchown):
        tmpdir = tempfile.mkdtemp()
        fn1 = os.path.join(tmpdir, 'test.conf')
        try:
            context = {'nginx_port': 80}
            templating.render('test.conf', fn1, context,
                              owner=pwd.getpwuid(os.getuid()).pw_name,
                              group=grp.getgrgid(os.getgid()).gr_name,
                              templates_dir=TEMPLATES_DIR)
            with open(fn1) as f:
                contents = f.read()

            self.assertRegexpMatches(contents, 'something')
        finally:
            shutil.rmtree(tmpdir, ignore_errors=True)

    @mock.patch.object(templating, 'hookenv')
    @mock.patch('jinja2.Environment')
    def test_load_error(self, Env, hookenv):
        Env().get_template.side_effect = jinja2.exceptions.TemplateNotFound(
            'fake_cc.yml')
        self.assertRaises(
            jinja2.exceptions.TemplateNotFound, templating.render,
            'fake.src', 'fake.tgt', {}, templates_dir='tmpl')
        hookenv.log.assert_called_once_with(
            'Could not load template fake.src from tmpl.', level=hookenv.ERROR)
